@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,13 +24,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_acceptor_epoll.c 2034 2007-09-25 08:54:11Z des $
+ * $Id: cache_acceptor_epoll.c 2972 2008-07-20 10:45:09Z phk $
  *
  * XXX: We need to pass sessions back into the event engine when they are
  * reused.  Not sure what the most efficient way is for that.  For now
  * write the session pointer to a pipe which the event engine monitors.
  */
 
+#include "config.h"
+
 #if defined(HAVE_EPOLL_CTL)
 
 #include <stdio.h>
@@ -41,7 +43,6 @@
 
 #include <sys/epoll.h>
 
-#include "heritage.h"
 #include "shmlog.h"
 #include "cache.h"
 #include "cache_acceptor.h"
@@ -70,9 +71,12 @@ vca_main(void *arg)
 {
 	struct epoll_event ev;
 	double deadline;
+	int dotimer = 0;
+	double last_timeout = 0, tmp_timeout;
 	struct sess *sp, *sp2;
 	int i;
 
+	THR_SetName("cache-epoll");
 	(void)arg;
 
 	epfd = epoll_create(16);
@@ -81,7 +85,7 @@ vca_main(void *arg)
 	vca_add(vca_pipes[0], vca_pipes);
 
 	while (1) {
-		if (epoll_wait(epfd, &ev, 1, 100) > 0) {
+		if ((dotimer = epoll_wait(epfd, &ev, 1, 100)) > 0) {
 			if (ev.data.ptr == vca_pipes) {
 				i = read(vca_pipes[0], &sp, sizeof sp);
 				assert(i == sizeof sp);
@@ -90,18 +94,22 @@ vca_main(void *arg)
 				vca_add(sp->fd, sp);
 			} else {
 				CAST_OBJ_NOTNULL(sp, ev.data.ptr, SESS_MAGIC);
-				i = vca_pollsession(sp);
-				if (i >= 0) {
+				i = HTC_Rx(sp->htc);
+				if (i != 0) {
 					VTAILQ_REMOVE(&sesshead, sp, list);
-					if (sp->fd != -1)
-						vca_del(sp->fd);
-					if (i == 0)
-						vca_handover(sp, i);
-					else
-						SES_Delete(sp);
+					vca_del(sp->fd);
+					vca_handover(sp, i);
 				}
 			}
 		}
+		tmp_timeout = TIM_mono();
+		if ((tmp_timeout - last_timeout) > 60) {
+			last_timeout = tmp_timeout;
+		} else {
+			if (dotimer > 0)
+				continue;
+		}
+
 		/* check for timeouts */
 		deadline = TIM_real() - params->sess_timeout;
 		VTAILQ_FOREACH_SAFE(sp, &sesshead, list, sp2) {
