@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_acceptor_kqueue.c 3580 2009-02-05 08:45:25Z tfheen $
+ * $Id: cache_acceptor_kqueue.c 3997 2009-03-23 14:40:52Z tfheen $
  *
  * XXX: We need to pass sessions back into the event engine when they are
  * reused.  Not sure what the most efficient way is for that.  For now
@@ -159,8 +159,10 @@ vca_kqueue_main(void *arg)
 	assert(kq >= 0);
 
 	j = 0;
-	EV_SET(&ke[j++], 0, EVFILT_TIMER, EV_ADD, 0, 100, NULL);
-	EV_SET(&ke[j++], vca_pipes[0], EVFILT_READ, EV_ADD, 0, 0, vca_pipes);
+	EV_SET(&ke[j], 0, EVFILT_TIMER, EV_ADD, 0, 100, NULL);
+	j++;
+	EV_SET(&ke[j], vca_pipes[0], EVFILT_READ, EV_ADD, 0, 0, vca_pipes);
+	j++;
 	AZ(kevent(kq, ke, j, NULL, 0, NULL));
 
 	nki = 0;
