@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,51 +26,50 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_fetch.c 2299 2007-12-18 16:55:32Z des $
+ * $Id: cache_fetch.c 3738 2009-02-11 07:36:03Z tfheen $
  */
 
+#include "config.h"
+
 #include <stdio.h>
-#include <ctype.h>
 #include <inttypes.h>
-#include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
 #include <fcntl.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netdb.h>
-#include <time.h>
 
 #include "shmlog.h"
 #include "cache.h"
-#include "heritage.h"
+#include "stevedore.h"
+#include "cli_priv.h"
+
+static unsigned fetchfrag;
 
 /*--------------------------------------------------------------------*/
 
 static int
-fetch_straight(const struct sess *sp, int fd, struct http *hp, char *b)
+fetch_straight(struct sess *sp, struct http_conn *htc, const char *b)
 {
 	int i;
 	unsigned char *p;
-	off_t	cl;
+	uintmax_t cll;
+	unsigned cl;
 	struct storage *st;
 
-	cl = strtoumax(b, NULL, 0);
-	if (cl == 0)
+	cll = strtoumax(b, NULL, 0);
+	if (cll == 0)
 		return (0);
 
-	st = STV_alloc(cl);
+	cl = (unsigned)cll;
+	assert((uintmax_t)cl == cll); /* Protect against bogusly large values */
+
+	st = STV_alloc(sp, cl);
 	VTAILQ_INSERT_TAIL(&sp->obj->store, st, list);
 	st->len = cl;
 	sp->obj->len = cl;
 	p = st->ptr;
 
-	i = fcntl(fd, F_GETFL);		/* XXX ? */
-	i &= ~O_NONBLOCK;
-	i = fcntl(fd, F_SETFL, i);
-
 	while (cl > 0) {
-		i = http_Read(hp, fd, p, cl);
+		i = HTC_Read(htc, p, cl);
 		if (i <= 0)
 			return (-1);
 		p += i;
@@ -83,12 +82,12 @@ fetch_straight(const struct sess *sp, int fd, struct http *hp, char *b)
 /* XXX: Cleanup.  It must be possible somehow :-( */
 
 static int
-fetch_chunked(const struct sess *sp, int fd, struct http *hp)
+fetch_chunked(struct sess *sp, struct http_conn *htc)
 {
 	int i;
 	char *q;
 	struct storage *st;
-	unsigned u, v;
+	unsigned u, v, w;
 	char buf[20];		/* XXX: arbitrary */
 	char *bp, *be;
 
@@ -111,9 +110,10 @@ fetch_chunked(const struct sess *sp, int fd, struct http *hp)
 
 		/* If we didn't succeed, add to buffer, try again */
 		if (q == NULL || q == buf || *q != '\n') {
-			xxxassert(be > bp);
+			if (bp >= be)
+				return (-1);
 			/*
-			 * The sematics we need here is "read until you have
+			 * The semantics we need here is "read until you have
 			 * received at least one character, but feel free to
 			 * return up to (be-bp) if they are available, but do
 			 * not wait for those extra characters.
@@ -127,7 +127,7 @@ fetch_chunked(const struct sess *sp, int fd, struct http *hp)
 			 * deal more complex than reading a single character
 			 * at a time.
 			 */
-			i = http_Read(hp, fd, bp, 1);
+			i = HTC_Read(htc, bp, 1);
 			if (i <= 0)
 				return (-1);
 			bp += i;
@@ -148,7 +148,7 @@ fetch_chunked(const struct sess *sp, int fd, struct http *hp)
 				v = u;
 				if (u < params->fetch_chunksize * 1024)
 					v = params->fetch_chunksize * 1024;
-				st = STV_alloc(v);
+				st = STV_alloc(sp, v);
 				VTAILQ_INSERT_TAIL(&sp->obj->store, st, list);
 			}
 			v = st->space - st->len;
@@ -156,17 +156,16 @@ fetch_chunked(const struct sess *sp, int fd, struct http *hp)
 				v = u;
 
 			/* Handle anything left in our buffer first */
-			i = bp - q;
-			assert(i >= 0);
-			if (i > v)
-				i = v;
-			if (i != 0) {
-				memcpy(st->ptr + st->len, q, i);
-				st->len += i;
-				sp->obj->len += i;
-				u -= i;
-				v -= i;
-				q += i;
+			w = pdiff(q, bp);
+			if (w > v)
+				w = v;
+			if (w != 0) {
+				memcpy(st->ptr + st->len, q, w);
+				st->len += w;
+				sp->obj->len += w;
+				u -= w;
+				v -= w;
+				q += w;
 			}
 			if (u == 0)
 				break;
@@ -175,7 +174,7 @@ fetch_chunked(const struct sess *sp, int fd, struct http *hp)
 
 			/* Pick up the rest of this chunk */
 			while (v > 0) {
-				i = http_Read(hp, fd, st->ptr + st->len, v);
+				i = HTC_Read(htc, st->ptr + st->len, v);
 				if (i <= 0)
 					return (-1);
 				st->len += i;
@@ -187,7 +186,7 @@ fetch_chunked(const struct sess *sp, int fd, struct http *hp)
 		assert(u == 0);
 
 		/* We might still have stuff in our buffer */
-		v = bp - q;
+		v = pdiff(q, bp);
 		if (v > 0)
 			memcpy(buf, q, v);
 		q = bp = buf + v;
@@ -204,33 +203,44 @@ fetch_chunked(const struct sess *sp, int fd, struct http *hp)
 
 /*--------------------------------------------------------------------*/
 
-#include <errno.h>
+static void
+dump_st(const struct sess *sp, const struct storage *st)
+{
+	txt t;
+
+	t.b = (void*)st->ptr;
+	t.e = (void*)(st->ptr + st->len);
+	WSLR(sp->wrk, SLT_Debug, sp->fd, t);
+}
 
 static int
-fetch_eof(const struct sess *sp, int fd, struct http *hp)
+fetch_eof(struct sess *sp, struct http_conn *htc)
 {
 	int i;
 	unsigned char *p;
 	struct storage *st;
 	unsigned v;
 
-	i = fcntl(fd, F_GETFL);		/* XXX ? */
-	i &= ~O_NONBLOCK;
-	i = fcntl(fd, F_SETFL, i);
-
+	if (fetchfrag > 0)
+		WSL(sp->wrk, SLT_Debug, sp->fd,
+		    "Fetch %d byte segments:", fetchfrag);
 	p = NULL;
 	v = 0;
 	st = NULL;
 	while (1) {
 		if (v == 0) {
-			st = STV_alloc(params->fetch_chunksize * 1024);
+			if (st != NULL && fetchfrag > 0)
+				dump_st(sp, st);
+			st = STV_alloc(sp, params->fetch_chunksize * 1024);
 			VTAILQ_INSERT_TAIL(&sp->obj->store, st, list);
 			p = st->ptr + st->len;
 			v = st->space - st->len;
+			if (fetchfrag > 0 && v > fetchfrag)
+				v = fetchfrag;
 		}
 		AN(p);
 		AN(st);
-		i = http_Read(hp, fd, p, v);
+		i = HTC_Read(htc, p, v);
 		if (i < 0)
 			return (-1);
 		if (i == 0)
@@ -240,6 +250,8 @@ fetch_eof(const struct sess *sp, int fd, struct http *hp)
 		st->len += i;
 		sp->obj->len += i;
 	}
+	if (fetchfrag > 0)
+		dump_st(sp, st);
 
 	if (st->len == 0) {
 		VTAILQ_REMOVE(&sp->obj->store, st, list);
@@ -250,6 +262,49 @@ fetch_eof(const struct sess *sp, int fd, struct http *hp)
 	return (1);
 }
 
+/*--------------------------------------------------------------------
+ * Fetch any body attached to the incoming request, and either write it
+ * to the backend (if we pass) or discard it (anything else).
+ * This is mainly a separate function to isolate the stack buffer and
+ * to contain the complexity when we start handling chunked encoding.
+ */
+
+int
+FetchReqBody(struct sess *sp)
+{
+	unsigned long content_length;
+	char buf[8192];
+	char *ptr, *endp;
+	int rdcnt;
+
+	if (http_GetHdr(sp->http, H_Content_Length, &ptr)) {
+
+		content_length = strtoul(ptr, &endp, 10);
+		/* XXX should check result of conversion */
+		while (content_length) {
+			if (content_length > sizeof buf)
+				rdcnt = sizeof buf;
+			else
+				rdcnt = content_length;
+			rdcnt = HTC_Read(sp->htc, buf, rdcnt);
+			if (rdcnt <= 0)
+				return (1);
+			content_length -= rdcnt;
+			if (!sp->sendbody)
+				continue;
+			(void)WRW_Write(sp->wrk, buf, rdcnt); /* XXX: stats ? */
+			if (WRW_Flush(sp->wrk))
+				return (2);
+		}
+	}
+	if (http_GetHdr(sp->http, H_Transfer_Encoding, NULL)) {
+		/* XXX: Handle chunked encoding. */
+		WSL(sp->wrk, SLT_Debug, sp->fd, "Transfer-Encoding in request");
+		return (1);
+	}
+	return (0);
+}
+
 /*--------------------------------------------------------------------*/
 
 int
@@ -262,12 +317,17 @@ Fetch(struct sess *sp)
 	struct http *hp, *hp2;
 	struct storage *st;
 	struct bereq *bereq;
-	int len, mklen, is_head;
+	int mklen, is_head;
+	struct http_conn htc[1];
+	int i;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
-	assert(sp->obj->busy != 0);
+	CHECK_OBJ_NOTNULL(sp->bereq, BEREQ_MAGIC);
+	AN(sp->director);
+	AN(sp->obj->busy);
+	AN(sp->bereq);
 	w = sp->wrk;
 	bereq = sp->bereq;
 	hp = bereq->http;
@@ -275,98 +335,127 @@ Fetch(struct sess *sp)
 
 	sp->obj->xid = sp->xid;
 
-	vc = VBE_GetFd(sp);
-	if (vc == NULL)
-		return (1);
-	WRK_Reset(w, &vc->fd);
-	http_Write(w, hp, 0);
-	if (WRK_Flush(w)) {
-		/* XXX: cleanup */
-		return (1);
+	/* Set up obj's workspace */
+	WS_Assert(sp->obj->ws_o);
+	VBE_GetFd(sp);
+	if (sp->vbe == NULL)
+		return (__LINE__);
+	vc = sp->vbe;
+	/* Inherit the backend timeouts from the selected backend */
+	SES_InheritBackendTimeouts(sp);
+
+	/*
+	 * Now that we know our backend, we can set a default Host:
+	 * header if one is necessary.
+	 * XXX: This possibly ought to go into the default VCL
+	 */
+	if (!http_GetHdr(hp, H_Host, &b))
+		VBE_AddHostHeader(sp);
+
+	TCP_blocking(vc->fd);	/* XXX: we should timeout instead */
+	WRW_Reserve(w, &vc->fd);
+	(void)http_Write(w, hp, 0);	/* XXX: stats ? */
+
+	/* Deal with any message-body the request might have */
+	i = FetchReqBody(sp);
+	if (WRW_FlushRelease(w) || i > 0) {
+		VBE_ClosedFd(sp);
+		/* XXX: other cleanup ? */
+		return (__LINE__);
 	}
 
+	/* Checkpoint the shmlog here */
+	WSL_Flush(w, 0);
+
 	/* XXX is this the right place? */
 	VSL_stats->backend_req++;
 
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->wrk, WORKER_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
-
-	if (http_RecvHead(hp, vc->fd)) {
-		/* XXX: cleanup */
-		return (1);
+	HTC_Init(htc, bereq->ws, vc->fd);
+	TCP_set_read_timeout(vc->fd, sp->first_byte_timeout);
+	do {
+		i = HTC_Rx(htc);
+		TCP_set_read_timeout(vc->fd, sp->between_bytes_timeout);
 	}
-	if (http_DissectResponse(sp->wrk, hp, vc->fd)) {
-		/* XXX: cleanup */
-		return (1);
+	while (i == 0);
+
+	if (i < 0) {
+		VBE_ClosedFd(sp);
+		/* XXX: other cleanup ? */
+		return (__LINE__);
 	}
 
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->wrk, WORKER_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
+	if (http_DissectResponse(sp->wrk, htc, hp)) {
+		VBE_ClosedFd(sp);
+		/* XXX: other cleanup ? */
+		return (__LINE__);
+	}
 
 	sp->obj->entered = TIM_real();
 
-	assert(sp->obj->busy != 0);
-
 	if (http_GetHdr(hp, H_Last_Modified, &b))
 		sp->obj->last_modified = TIM_parse(b);
 
 	/* Filter into object */
-	hp2 = &sp->obj->http;
-	len = hp->rx_e - hp->rx_s;
-	len += 256;		/* margin for content-length etc */
-
-	b = malloc(len);
-	AN(b);
-	http_Setup(hp2, b, len);
+	hp2 = sp->obj->http;
 
 	hp2->logtag = HTTP_Obj;
 	http_CopyResp(hp2, hp);
 	http_FilterFields(sp->wrk, sp->fd, hp2, hp, HTTPH_A_INS);
 	http_CopyHome(sp->wrk, sp->fd, hp2);
 
-
 	/* Determine if we have a body or not */
 	cls = 0;
 	mklen = 0;
 	if (is_head) {
 		/* nothing */
 	} else if (http_GetHdr(hp, H_Content_Length, &b)) {
-		cls = fetch_straight(sp, vc->fd, hp, b);
+		cls = fetch_straight(sp, htc, b);
 		mklen = 1;
 	} else if (http_HdrIs(hp, H_Transfer_Encoding, "chunked")) {
-		cls = fetch_chunked(sp, vc->fd, hp);
+		cls = fetch_chunked(sp, htc);
 		mklen = 1;
 	} else if (http_GetHdr(hp, H_Transfer_Encoding, &b)) {
 		/* XXX: AUGH! */
-		VSL(SLT_Debug, vc->fd, "Invalid Transfer-Encoding");
-		VBE_ClosedFd(sp->wrk, vc);
-		return (-1);
-	} else if (strcmp(http_GetProto(hp), "HTTP/1.1")) {
-		switch (http_GetStatus(hp)) {
-			case 200:
-				cls = fetch_eof(sp, vc->fd, hp);
-				mklen = 1;
-				break;
-			default:
-				break;
-		}
+		WSL(sp->wrk, SLT_Debug, vc->fd, "Invalid Transfer-Encoding");
+		VBE_ClosedFd(sp);
+		return (__LINE__);
+	} else if (http_HdrIs(hp, H_Connection, "keep-alive")) {
+		/*
+		 * If we have Connection: keep-alive, it cannot possibly be
+		 * EOF encoded, and since it is neither length nor chunked
+		 * it must be zero length.
+		 */
+		mklen = 1;
+	} else if (http_HdrIs(hp, H_Connection, "close")) {
+		/*
+		 * If we have connection closed, it is safe to read what
+		 * comes in any case.
+		 */
+		cls = fetch_eof(sp, htc);
+		mklen = 1;
+	} else if (hp->protover < 1.1) {
+		/*
+		 * With no Connection header, assume EOF
+		 */
+		cls = fetch_eof(sp, htc);
+		mklen = 1;
+	} else {
+		/*
+		 * Assume zero length
+		 */
+		mklen = 1;
 	}
 
-	if (mklen > 0)
-		http_PrintfHeader(sp->wrk, sp->fd, hp2,
-		    "Content-Length: %u", sp->obj->len);
-
 	if (cls < 0) {
+		/* XXX: Wouldn't this store automatically be released ? */
 		while (!VTAILQ_EMPTY(&sp->obj->store)) {
 			st = VTAILQ_FIRST(&sp->obj->store);
 			VTAILQ_REMOVE(&sp->obj->store, st, list);
 			STV_free(st);
 		}
-		VBE_ClosedFd(sp->wrk, vc);
+		VBE_ClosedFd(sp);
 		sp->obj->len = 0;
-		return (-1);
+		return (__LINE__);
 	}
 
 	{
@@ -379,13 +468,48 @@ Fetch(struct sess *sp)
 		assert(uu == sp->obj->len);
 	}
 
-	if (http_GetHdr(hp, H_Connection, &b) && !strcasecmp(b, "close"))
+	if (mklen > 0)
+		http_PrintfHeader(sp->wrk, sp->fd, hp2,
+		    "Content-Length: %u", sp->obj->len);
+
+	if (http_HdrIs(hp, H_Connection, "close")) 
 		cls = 1;
 
 	if (cls)
-		VBE_ClosedFd(sp->wrk, vc);
+		VBE_ClosedFd(sp);
 	else
-		VBE_RecycleFd(sp->wrk, vc);
+		VBE_RecycleFd(sp);
 
 	return (0);
 }
+
+/*--------------------------------------------------------------------
+ * Debugging aids
+ */
+
+static void
+debug_fragfetch(struct cli *cli, const char * const *av, void *priv)
+{
+	(void)priv;
+	(void)cli;
+	fetchfrag = strtoul(av[2], NULL, 0);
+}
+
+static struct cli_proto debug_cmds[] = {
+	{ "debug.fragfetch", "debug.fragfetch",
+		"\tEnable fetch fragmentation\n", 1, 1, debug_fragfetch },
+	{ NULL }
+};
+
+/*--------------------------------------------------------------------
+ *
+ */
+
+void
+Fetch_Init(void)
+{
+
+	CLI_AddFuncs(DEBUG_CLI, debug_cmds);
+}
+
+
