@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_panic.c 3672 2009-02-06 13:47:53Z tfheen $
+ * $Id: cache_panic.c 3980 2009-03-23 08:31:25Z tfheen $
  */
 
 #include "config.h"
@@ -260,7 +260,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	switch(xxx) {
 	case 3:
 		vsb_printf(vsp,
-		    "Wrong turn:\n%s\n", cond);
+		    "Wrong turn at %s:%d:\n%s\n", file, line, cond);
 		break;
 	case 2:
 		vsb_printf(vsp,
