@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,11 +26,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_pipe.c 2149 2007-10-19 11:46:03Z des $
+ * $Id: cache_pipe.c 3717 2009-02-10 14:20:29Z tfheen $
  *
  * XXX: charge bytes to srcaddr
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -39,7 +41,6 @@
 #include <sys/socket.h>
 
 #include "shmlog.h"
-#include "heritage.h"
 #include "cache.h"
 
 static int
@@ -55,10 +56,8 @@ rdf(int fd0, int fd1)
 		j = write(fd1, p, i);
 		if (j <= 0)
 			return (1);
-		if (i != j) {
-			printf("flunk %d %d\n", i, j);
-			usleep(100000);		/* XXX hack */
-		}
+		if (i != j)
+			(void)usleep(100000);		/* XXX hack */
 	}
 	return (0);
 }
@@ -67,7 +66,6 @@ void
 PipeSession(struct sess *sp)
 {
 	struct vbe_conn *vc;
-	char *b, *e;
 	struct worker *w;
 	struct bereq *bereq;
 	struct pollfd fds[2];
@@ -79,25 +77,28 @@ PipeSession(struct sess *sp)
 	bereq = sp->bereq;
 	sp->bereq = NULL;
 
-	vc = VBE_GetFd(sp);
-	if (vc == NULL)
+	VBE_GetFd(sp);
+	if (sp->vbe == NULL)
 		return;
+	vc = sp->vbe;
+	TCP_blocking(vc->fd);
 
-	WRK_Reset(w, &vc->fd);
-	http_Write(w, bereq->http, 0);
+	WRW_Reserve(w, &vc->fd);
+	sp->acct_req.hdrbytes += http_Write(w, bereq->http, 0);
 
-	if (http_GetTail(sp->http, 0, &b, &e) && b != e)
-		WRK_Write(w, b, e - b);
+	if (sp->htc->pipeline.b != NULL)
+		sp->acct_req.bodybytes +=
+		    WRW_Write(w, sp->htc->pipeline.b, Tlen(sp->htc->pipeline));
 
-	if (WRK_Flush(w)) {
+	i = WRW_FlushRelease(w);
+	VBE_free_bereq(&bereq);
+
+	if (i) {
 		vca_close_session(sp, "pipe");
-		VBE_ClosedFd(sp->wrk, vc);
+		VBE_ClosedFd(sp);
 		return;
 	}
 
-	vbe_free_bereq(bereq);
-	bereq = NULL;
-
 	sp->t_resp = TIM_real();
 
 	memset(fds, 0, sizeof fds);
@@ -110,29 +111,29 @@ PipeSession(struct sess *sp)
 		fds[0].revents = 0;
 		fds[1].revents = 0;
 		i = poll(fds, 2, params->pipe_timeout * 1000);
-		if (i < 1) 
+		if (i < 1)
 			break;
 		if (fds[0].revents && rdf(vc->fd, sp->fd)) {
-			shutdown(vc->fd, SHUT_RD);
-			shutdown(sp->fd, SHUT_WR);
+			(void)shutdown(vc->fd, SHUT_RD);
+			(void)shutdown(sp->fd, SHUT_WR);
 			fds[0].events = 0;
 			fds[0].fd = -1;
 		}
 		if (fds[1].revents && rdf(sp->fd, vc->fd)) {
-			shutdown(sp->fd, SHUT_RD);
-			shutdown(vc->fd, SHUT_WR);
+			(void)shutdown(sp->fd, SHUT_RD);
+			(void)shutdown(vc->fd, SHUT_WR);
 			fds[1].events = 0;
 			fds[1].fd = -1;
 		}
 	}
 	if (fds[0].fd >= 0) {
-		shutdown(vc->fd, SHUT_RD);
-		shutdown(sp->fd, SHUT_WR);
+		(void)shutdown(vc->fd, SHUT_RD);
+		(void)shutdown(sp->fd, SHUT_WR);
 	}
 	if (fds[1].fd >= 0) {
-		shutdown(sp->fd, SHUT_RD);
-		shutdown(vc->fd, SHUT_WR);
+		(void)shutdown(sp->fd, SHUT_RD);
+		(void)shutdown(vc->fd, SHUT_WR);
 	}
 	vca_close_session(sp, "pipe");
-	VBE_ClosedFd(sp->wrk, vc);
+	VBE_ClosedFd(sp);
 }
