@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_pool.c 3645 2009-02-06 10:28:54Z tfheen $
+ * $Id: cache_pool.c 3999 2009-03-23 14:51:24Z tfheen $
  *
  * We maintain a number of worker thread pools, to spread lock contention.
  *
@@ -48,7 +48,7 @@
 #include <sys/uio.h>
 
 #ifdef SENDFILE_WORKS
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__DragonFly__)
 #include <sys/socket.h>
 #elif defined(__linux__)
 #include <sys/sendfile.h>
@@ -208,7 +208,7 @@ WRW_Sendfile(struct worker *w, int fd, off_t off, unsigned len)
 	assert(fd >= 0);
 	assert(len > 0);
 
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__DragonFly__)
 	do {
 		struct sf_hdtr sfh;
 		memset(&sfh, 0, sizeof sfh);
@@ -486,7 +486,7 @@ wrk_decimate_flock(struct wq *qp, double t_idle, struct varnish_stats *vs)
 	if (w != NULL) {
 		AZ(w->wrq);
 		AZ(pthread_cond_signal(&w->cond));
-		(void)usleep(params->wthread_purge_delay * 1000);
+		TIM_sleep(params->wthread_purge_delay * 1e-3);
 	}
 }
 
@@ -542,7 +542,7 @@ wrk_herdtimer_thread(void *priv)
 		VSL_stats->n_wrk_drop = vs->n_wrk_drop;
 		VSL_stats->n_wrk_overflow = vs->n_wrk_overflow;
 
-		(void)usleep(params->wthread_purge_delay * 1000);
+		TIM_sleep(params->wthread_purge_delay * 1e-3);
 	}
 }
 
@@ -568,11 +568,11 @@ wrk_breed_flock(struct wq *qp)
 			VSL(SLT_Debug, 0, "Create worker thread failed %d %s",
 			    errno, strerror(errno));
 			VSL_stats->n_wrk_failed++;
-			(void)usleep(params->wthread_fail_delay * 1000);
+			TIM_sleep(params->wthread_fail_delay * 1e-3);
 		} else {
 			AZ(pthread_detach(tp));
 			VSL_stats->n_wrk_create++;
-			(void)usleep(params->wthread_add_delay * 1000);
+			TIM_sleep(params->wthread_add_delay * 1e-3);
 		}
 	}
 	qp->lqueue = qp->nqueue;
