@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_response.c 3701 2009-02-09 12:32:54Z tfheen $
+ * $Id: cache_response.c 3995 2009-03-23 14:23:31Z tfheen $
  */
 
 #include "config.h"
@@ -152,7 +152,9 @@ RES_WriteObj(struct sess *sp)
 	}
 
 	if (sp->wantbody) {
-		if (sp->esis > 0 && sp->http->protover >= 1.1) {
+		if (sp->esis > 0 &&
+		    sp->http->protover >= 1.1 &&
+		    sp->obj->len > 0) {
 			sprintf(lenbuf, "%x\r\n", sp->obj->len);
 			(void)WRW_Write(sp->wrk, lenbuf, -1);
 		}
@@ -181,7 +183,9 @@ RES_WriteObj(struct sess *sp)
 			(void)WRW_Write(sp->wrk, st->ptr, st->len);
 		}
 		assert(u == sp->obj->len);
-		if (sp->esis > 0 && sp->http->protover >= 1.1)
+		if (sp->esis > 0 &&
+		    sp->http->protover >= 1.1 &&
+		    sp->obj->len > 0)
 			(void)WRW_Write(sp->wrk, "\r\n", -1);
 	}
 	if (WRW_FlushRelease(sp->wrk))
