@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_vrt.c 3726 2009-02-10 15:06:31Z tfheen $
+ * $Id: cache_vrt.c 4015 2009-04-01 12:21:48Z tfheen $
  *
  * Runtime support for compiled VCL programs
  */
@@ -54,6 +54,7 @@
 #include "cache_backend.h"
 
 void *vrt_magic_string_end = &vrt_magic_string_end;
+char vrt_hostname[255] = "";
 
 /*--------------------------------------------------------------------*/
 
@@ -62,7 +63,8 @@ VRT_error(struct sess *sp, unsigned code, const char *reason)
 {
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	WSL(sp->wrk, SLT_Debug, 0, "VCL_error(%u, %s)", code, reason);
+	WSL(sp->wrk, SLT_Debug, 0, "VCL_error(%u, %s)", code, reason ? 
+	    reason : "(null)");
 	sp->err_code = code ? code : 503;
 	sp->err_reason = reason ? reason : http_StatusMessage(sp->err_code);
 }
@@ -564,6 +566,29 @@ VRT_r_server_ip(struct sess *sp)
 	return (sp->mysockaddr);
 }
 
+const char*
+VRT_r_server_identity(struct sess *sp)
+{
+	(void)sp;
+
+	if (heritage.identity[0] != '\0')
+		return heritage.identity;
+	else 
+		return heritage.name;
+}
+
+
+const char*
+VRT_r_server_hostname(struct sess *sp)
+{
+	(void)sp;
+
+	if (vrt_hostname[0] == '\0')
+		gethostname(vrt_hostname, 255);
+	
+	return (const char*) &vrt_hostname;
+}
+
 int
 VRT_r_server_port(struct sess *sp)
 {
