@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_vrt_esi.c 3744 2009-02-11 08:02:31Z tfheen $
+ * $Id: cache_vrt_esi.c 4013 2009-03-31 07:44:56Z tfheen $
  *
  * Runtime support for compiled VCL programs ESI processing.
  *
@@ -71,6 +71,7 @@ VTAILQ_HEAD(esibithead, esi_bit);
 
 struct esi_ptr {
 	const char		*p;
+	const char              *e;
 	struct storage		*st;
 };
 
@@ -108,9 +109,11 @@ Nep(struct esi_ptr *ep)
 	ep->st = VTAILQ_NEXT(ep->st, list);
 	if (ep->st != NULL) {
 		ep->p = (char *)ep->st->ptr;
+		ep->e = ep->p + ep->st->len;
 		return;
 	}
 	ep->p = finis;
+	ep->e = finis;
 	return;
 }
 
@@ -118,7 +121,7 @@ static void
 N(struct esi_work *ew)
 {
 
-	if (*ew->p.p != '\0')
+	if (ew->p.p < ew->p.e)
 		ew->off++;
 	Nep(&ew->p);
 }
@@ -512,7 +515,7 @@ parse_comment(struct esi_work *ew)
 			N(ew);
 			break;
 		}
-	} while (*ew->p.p != '\0');
+	} while (ew->p.p < ew->p.e);
 }
 
 /*--------------------------------------------------------------------*/
@@ -531,7 +534,7 @@ parse_cdata(struct esi_work *ew)
 			N(ew);
 			break;
 		}
-	} while (*ew->p.p != '\0');
+	} while (ew->p.p < ew->p.e);
 }
 
 /*--------------------------------------------------------------------*/
@@ -547,8 +550,8 @@ parse_esi_tag(struct esi_work *ew, int closing)
 
 	do
 		N(ew);
-	while (*ew->p.p != '>' && *ew->p.p != '\0');
-	if (*ew->p.p == '\0') {
+	while (*ew->p.p != '>' && ew->p.p < ew->p.e);
+	if (ew->p.p == ew->p.e) {
 		esi_addpfx(ew);
 		esi_error(ew, ew->s.p, 0,
 		    "XML 1.0 incomplete language element");
@@ -683,11 +686,12 @@ VRT_ESI(struct sess *sp)
 	ew->p.st = VTAILQ_FIRST(&sp->obj->store);
 	AN(ew->p.st);
 	ew->p.p = (char *)ew->p.st->ptr;
-
+	ew->p.e = ew->p.p + ew->p.st->len;
+	
 	/* ->s points to the first un-dealt-with byte */
 	ew->s = ew->p;
 
-	while (*ew->p.p != '\0') {
+	while (ew->p.p < ew->p.e) {
 
 		if (ew->incmt && *ew->p.p == '-' && !CMP(&ew->p, "-->")) {
 			/* End of ESI comment */
@@ -724,7 +728,7 @@ VRT_ESI(struct sess *sp)
 				/* XXX: drop this ? */
 				do {
 					N(ew);
-				} while (*ew->p.p != '>' && *ew->p.p != '\0');
+				} while (*ew->p.p != '>' && ew->p.p < ew->p.e);
 			}
 		}
 	}
