@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,11 +26,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_vrt_re.c 1693 2007-07-13 14:54:51Z des $
+ * $Id: cache_vrt_re.c 3650 2009-02-06 11:27:55Z tfheen $
  *
  * Runtime support for compiled VCL programs, regexps
  */
 
+#include "config.h"
+
 #include <sys/types.h>
 
 #include <stdio.h>
@@ -41,7 +43,6 @@
 
 #include "shmlog.h"
 #include "vrt.h"
-#include "vsb.h"
 #include "vcl.h"
 #include "cache.h"
 
@@ -53,7 +54,7 @@ VRT_re_init(void **rep, const char *re, int sub)
 	t = calloc(sizeof *t, 1);
 	XXXAN(t);
 	/* This was already check-compiled by the VCL compiler */
-	AZ(regcomp(t, re, REG_EXTENDED | (sub ? 0 : REG_NOSUB)));
+	AZ(regcomp(t, re, REG_EXTENDED | REG_ICASE | (sub ? 0 : REG_NOSUB)));
 	*rep = t;
 }
 
@@ -82,35 +83,21 @@ VRT_re_match(const char *s, void *re)
 	return (0);
 }
 
-int
-VRT_re_test(struct vsb *sb, const char *re, int sub)
-{
-	int i;
-	regex_t	t;
-	char buf[BUFSIZ];
-
-	memset(&t, 0, sizeof t);
-	i = regcomp(&t, re, REG_EXTENDED | (sub ? 0 : REG_NOSUB));
-	if (i == 0) {
-		regfree(&t);
-		return (0);
-	}
-	(void)regerror(i, &t, buf, sizeof buf);
-	vsb_printf(sb, "Regexp compilation error:\n\n%s\n\n", buf);
-	regfree(&t);
-	return (1);
-}
-
 const char *
-VRT_regsub(struct sess *sp, const char *str, void *re, const char *sub)
+VRT_regsub(const struct sess *sp, int all, const char *str, void *re,
+    const char *sub)
 {
 	regmatch_t pm[10];
 	regex_t *t;
 	int i, l;
-	char *b, *p, *e;
+	txt res;
+	char *b0;
+	const char *s;
 	unsigned u, x;
 
 	AN(re);
+	if (str == NULL)
+		return ("");
 	t = re;
 	i = regexec(t, str, 10, pm, 0);
 
@@ -119,54 +106,44 @@ VRT_regsub(struct sess *sp, const char *str, void *re, const char *sub)
 		return(str);
 
 	u = WS_Reserve(sp->http->ws, 0);
-	e = p = b = sp->http->ws->f;
-	e += u;
+	res.e = res.b = b0 = sp->http->ws->f;
+	res.e += u;
 
-	/* Copy prefix to match */
-	if (pm[0].rm_so > 0) {
-		if (p + pm[0].rm_so < e)
-			memcpy(p, str, pm[0].rm_so);
-		p += pm[0].rm_so;
-	}
+	do {
+		/* Copy prefix to match */
+		Tadd(&res, str, pm[0].rm_so);
 
-	for ( ; *sub != '\0'; sub++ ) {
-		if (*sub == '&') {
-			l = pm[0].rm_eo - pm[0].rm_so;
-			if (l > 0) {
-				if (p + l < e)
-					memcpy(p, str + pm[0].rm_so, l);
-				p += l;
+		for (s = sub ; *s != '\0'; s++ ) {
+			if (*s != '\\' || s[1] == '\0') {
+				if (res.b < res.e)
+					*res.b++ = *s;
+				continue;
 			}
-		} else if (*sub == '$' && isdigit(sub[1])) {
-			x = sub[1] - '0';
-			sub++;
-			l = pm[x].rm_eo - pm[x].rm_so;
-			if (l > 0) {
-				if (p + l < e)
-					memcpy(p, str + pm[x].rm_so, l);
-				p += l;
+			s++;
+			if (isdigit(*s)) {
+				x = *s - '0';
+				l = pm[x].rm_eo - pm[x].rm_so;
+				Tadd(&res, str + pm[x].rm_so, l);
+				continue;
+			} else {
+				if (res.b < res.e)
+					*res.b++ = *s;
 			}
-		} else {
-			if (p + 1 < e)
-				*p = *sub;
-			p++;
 		}
-	}
+		str += pm[0].rm_eo;
+		if (!all)
+			break;
+		i = regexec(t, str, 10, pm, 0);
+	} while (i != REG_NOMATCH);
 
 	/* Copy suffix to match */
-	l = strlen(str + pm[0].rm_eo);
-	if (l > 0) {
-		if (p + l < e)
-			memcpy(p, str + pm[0].rm_eo, l);
-		p += l;
-	}
-	if (p + 1 < e)
-		*p++ = '\0';
-	xxxassert(p <= e);
-	if (p > e) {
+	l = strlen(str) + 1;
+	Tadd(&res, str, l);
+	if (res.b >= res.e) {
 		WS_Release(sp->http->ws, 0);
 		return (str);
-	} 
-	WS_Release(sp->http->ws, p - b);
-	return (b);
+	}
+	Tcheck(res);
+	WS_ReleaseP(sp->http->ws, res.b);
+	return (b0);
 }
