@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: mgt.h 1572 2007-06-25 21:12:17Z phk $
+ * $Id: mgt.h 3732 2009-02-11 07:13:12Z tfheen $
  */
 
 #include "common.h"
@@ -36,20 +36,20 @@
 
 struct cli;
 
-extern struct evbase	*mgt_evb;
+extern struct vev_base	*mgt_evb;
 
 /* mgt_child.c */
 void mgt_run(int dflag, const char *T_arg);
 extern pid_t mgt_pid, child_pid;
+void mgt_stop_child(void);
 
 /* mgt_cli.c */
 
-void mgt_cli_init(void);
-void mgt_cli_setup(int fdi, int fdo, int verbose);
+void mgt_cli_setup(int fdi, int fdo, int verbose, const char *ident);
 int mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...);
 void mgt_cli_start_child(int fdi, int fdo);
 void mgt_cli_stop_child(void);
-int mgt_cli_telnet(const char *T_arg);
+void mgt_cli_telnet(int dflag, const char *T_arg);
 
 /* mgt_param.c */
 void MCF_ParamSync(void);
@@ -58,16 +58,21 @@ void MCF_ParamSet(struct cli *, const char *param, const char *val);
 
 /* mgt_vcc.c */
 void mgt_vcc_init(void);
-int mgt_vcc_default(const char *bflag, const char *fflag, int f_fd, int Cflag);
+int mgt_vcc_default(const char *bflag, char *vcl, int Cflag);
 int mgt_push_vcls_and_start(unsigned *status, char **p);
+int mgt_has_vcl(void);
+extern char *mgt_cc_cmd;
 
-#include "stevedore.h"
 
-extern struct stevedore sma_stevedore;
-extern struct stevedore smf_stevedore;
+#define REPORT0(pri, fmt)				\
+	do {						\
+		fprintf(stderr, fmt "\n");		\
+		syslog(pri, fmt);			\
+	} while (0)
 
-#include "hash_slinger.h"
-
-extern struct hash_slinger hsl_slinger;
-extern struct hash_slinger hcl_slinger;
+#define REPORT(pri, fmt, ...)				\
+	do {						\
+		fprintf(stderr, fmt "\n", __VA_ARGS__);	\
+		syslog(pri, fmt, __VA_ARGS__);		\
+	} while (0)
 
