@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: mgt_cli.c 3732 2009-02-11 07:13:12Z tfheen $
+ * $Id: mgt_cli.c 3981 2009-03-23 08:47:37Z tfheen $
  *
  * The management process' CLI handling
  */
@@ -372,6 +372,10 @@ mgt_cli_setup(int fdi, int fdo, int verbose, const char *ident)
 	cp->cli->sb = vsb_newauto();
 	XXXAN(cp->cli->sb);
 
+	/* Deal with TELNET options */
+	if (cp->fdi != 0)
+		VLU_SetTelnet(cp->vlu, cp->fdo);
+
 	cp->ev = calloc(sizeof *cp->ev, 1);
 	cp->ev->name = cp->name;
 	cp->ev->fd = fdi;
