@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: rfc2616.c 3732 2009-02-11 07:13:12Z tfheen $
+ * $Id: rfc2616.c 3993 2009-03-23 12:59:52Z tfheen $
  */
 
 #include "config.h"
@@ -197,7 +197,7 @@ RFC2616_cache_policy(const struct sess *sp, const struct http *hp)
 	}
 
 	sp->obj->ttl = RFC2616_Ttl(sp, hp, sp->obj);
-	if (sp->obj->ttl == 0)
+	if (sp->obj->ttl == 0. && sp->obj->grace == 0.)
 		sp->obj->cacheable = 0;
 
 	return (body);
