@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,9 +26,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: shmlog.c 1693 2007-07-13 14:54:51Z des $
+ * $Id: shmlog.c 3739 2009-02-11 07:41:41Z tfheen $
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -41,8 +43,6 @@
 #include "shmlog.h"
 #include "cache.h"
 
-#include "heritage.h"
-
 #define LOCKSHM(foo)					\
 	do {						\
 		if (pthread_mutex_trylock(foo)) {	\
@@ -64,12 +64,8 @@
 struct varnish_stats *VSL_stats;
 static struct shmloghead *loghead;
 static unsigned char *logstart;
-static MTX vsl_mtx;
+static pthread_mutex_t vsl_mtx;
 
-/*
- * This variant copies a byte-range directly to the log, without
- * taking the detour over sprintf()
- */
 
 static void
 vsl_wrap(void)
@@ -78,25 +74,44 @@ vsl_wrap(void)
 	*logstart = SLT_ENDMARKER;
 	logstart[loghead->ptr] = SLT_WRAPMARKER;
 	loghead->ptr = 0;
+	VSL_stats->shm_cycles++;
 }
 
-/*--------------------------------------------------------------------*/
+static void
+vsl_hdr(enum shmlogtag tag, unsigned char *p, unsigned len, unsigned id)
+{
 
-void
-VSLR(enum shmlogtag tag, unsigned id, const char *b, const char *e)
+	assert(len < 0x10000);
+	assert(id < 0x10000);
+	p[__SHMLOG_LEN_HIGH] = (len >> 8) & 0xff;
+	p[__SHMLOG_LEN_LOW] = len & 0xff;
+	p[__SHMLOG_ID_HIGH] = (id >> 8) & 0xff;
+	p[__SHMLOG_ID_LOW] = id & 0xff;
+	p[SHMLOG_DATA + len] = '\0';
+	p[SHMLOG_NEXTTAG + len] = SLT_ENDMARKER;
+	/* XXX: Write barrier here */
+	p[SHMLOG_TAG] = tag;
+}
+
+/*--------------------------------------------------------------------
+ * This variant copies a byte-range directly to the log, without
+ * taking the detour over sprintf()
+ */
+
+static void
+VSLR(enum shmlogtag tag, int id, txt t)
 {
 	unsigned char *p;
-	unsigned l;
+	unsigned l, mlen;
 
-	assert(b != NULL);
-	if (e == NULL)
-		e = strchr(b, '\0');
+	Tcheck(t);
+	mlen = params->shm_reclen;
 
 	/* Truncate */
-	l = e - b;
-	if (l > 255) {
-		l = 255;
-		e = b + l;
+	l = Tlen(t);
+	if (l > mlen) {
+		l = mlen;
+		t.e = t.b + l;
 	}
 
 	/* Only hold the lock while we find our space */
@@ -106,176 +121,172 @@ VSLR(enum shmlogtag tag, unsigned id, const char *b, const char *e)
 	assert(loghead->ptr < loghead->size);
 
 	/* Wrap if necessary */
-	if (loghead->ptr + 5 + l + 1 >= loghead->size)
+	if (loghead->ptr + SHMLOG_NEXTTAG + l + 1 >= loghead->size)
 		vsl_wrap();
 	p = logstart + loghead->ptr;
-	loghead->ptr += 5 + l;
-	p[5 + l] = SLT_ENDMARKER;
+	loghead->ptr += SHMLOG_NEXTTAG + l;
 	assert(loghead->ptr < loghead->size);
 	UNLOCKSHM(&vsl_mtx);
 
-	p[1] = l & 0xff;
-	p[2] = (id >> 8) & 0xff;
-	p[3] = id & 0xff;
-	memcpy(p + 4, b, l);
-	p[4 + l] = '\0';
-	/* XXX: memory barrier */
-	p[0] = tag;
+	memcpy(p + SHMLOG_DATA, t.b, l);
+	vsl_hdr(tag, p, l, id);
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-VSL(enum shmlogtag tag, unsigned id, const char *fmt, ...)
+VSL(enum shmlogtag tag, int id, const char *fmt, ...)
 {
 	va_list ap;
 	unsigned char *p;
-	unsigned n;
+	unsigned n, mlen;
+	txt t;
 
+	AN(fmt);
 	va_start(ap, fmt);
+	mlen = params->shm_reclen;
 
 	if (strchr(fmt, '%') == NULL) {
-		VSLR(tag, id, fmt, NULL);
-		return;
+		t.b = TRUST_ME(fmt);
+		t.e = strchr(t.b, '\0');
+		VSLR(tag, id, t);
+	} else {
+		LOCKSHM(&vsl_mtx);
+		VSL_stats->shm_writes++;
+		VSL_stats->shm_records++;
+		assert(loghead->ptr < loghead->size);
+
+		/* Wrap if we cannot fit a full size record */
+		if (loghead->ptr + SHMLOG_NEXTTAG + mlen + 1 >= loghead->size)
+			vsl_wrap();
+
+		p = logstart + loghead->ptr;
+		/* +1 for the NUL */
+		n = vsnprintf((char *)(p + SHMLOG_DATA), mlen + 1, fmt, ap);
+		if (n > mlen)
+			n = mlen;		/* we truncate long fields */
+		vsl_hdr(tag, p, n, id);
+		loghead->ptr += SHMLOG_NEXTTAG + n;
+		assert(loghead->ptr < loghead->size);
+		UNLOCKSHM(&vsl_mtx);
 	}
-
-	LOCKSHM(&vsl_mtx);
-	VSL_stats->shm_writes++;
-	VSL_stats->shm_records++;
-	assert(loghead->ptr < loghead->size);
-
-	/* Wrap if we cannot fit a full size record */
-	if (loghead->ptr + 5 + 255 + 1 >= loghead->size)
-		vsl_wrap();
-
-	p = logstart + loghead->ptr;
-	n = 0;
-	if (fmt != NULL) {
-		n = vsnprintf((char *)(p + 4), 256, fmt, ap);
-		if (n > 255)
-			n = 255; 	/* we truncate long fields */
-	}
-	p[1] = n & 0xff;
-	p[2] = (id >> 8) & 0xff;
-	p[3] = id & 0xff;
-	p[4 + n] = '\0';;
-	p[5 + n] = SLT_ENDMARKER;
-	p[0] = tag;
-
-	loghead->ptr += 5 + n;
-	assert(loghead->ptr < loghead->size);
-
-	UNLOCKSHM(&vsl_mtx);
-
 	va_end(ap);
 }
 
-
 /*--------------------------------------------------------------------*/
 
 void
-WSL_Flush(struct worker *w)
+WSL_Flush(struct worker *w, int overflow)
 {
 	unsigned char *p;
 	unsigned l;
 
-	if (w->wlp == w->wlog)
+	l = pdiff(w->wlb, w->wlp);
+	if (l == 0)
 		return;
-	l = w->wlp - w->wlog;
 	LOCKSHM(&vsl_mtx);
+	VSL_stats->shm_flushes += overflow;
 	VSL_stats->shm_writes++;
 	VSL_stats->shm_records += w->wlr;
 	if (loghead->ptr + l + 1 >= loghead->size)
 		vsl_wrap();
 	p = logstart + loghead->ptr;
-	memcpy(p + 1, w->wlog + 1, l - 1);
 	p[l] = SLT_ENDMARKER;
 	loghead->ptr += l;
 	assert(loghead->ptr < loghead->size);
-	p[0] = w->wlog[0];
 	UNLOCKSHM(&vsl_mtx);
-	w->wlp = w->wlog;
+	memcpy(p + 1, w->wlb + 1, l - 1);
+	/* XXX: memory barrier here */
+	p[0] = w->wlb[0];
+	w->wlp = w->wlb;
 	w->wlr = 0;
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-WSLR(struct worker *w, enum shmlogtag tag, unsigned id, const char *b, const char *e)
+WSLR(struct worker *w, enum shmlogtag tag, int id, txt t)
 {
 	unsigned char *p;
-	unsigned l;
+	unsigned l, mlen;
 
-	assert(b != NULL);
-	if (e == NULL)
-		e = strchr(b, '\0');
+	Tcheck(t);
+	mlen = params->shm_reclen;
 
 	/* Truncate */
-	l = e - b;
-	if (l > 255) {
-		l = 255;
-		e = b + l;
+	l = Tlen(t);
+	if (l > mlen) {
+		l = mlen;
+		t.e = t.b + l;
 	}
 
 	assert(w->wlp < w->wle);
 
 	/* Wrap if necessary */
-	if (w->wlp + 5 + l + 1 >= w->wle)
-		WSL_Flush(w);
+	if (w->wlp + SHMLOG_NEXTTAG + l + 1 >= w->wle)
+		WSL_Flush(w, 1);
 	p = w->wlp;
-	w->wlp += 5 + l;
+	w->wlp += SHMLOG_NEXTTAG + l;
 	assert(w->wlp < w->wle);
-	p[5 + l] = SLT_ENDMARKER;
-
-	p[1] = l & 0xff;
-	p[2] = (id >> 8) & 0xff;
-	p[3] = id & 0xff;
-	memcpy(p + 4, b, l);
-	p[4 + l] = '\0';
-	p[0] = tag;
+	memcpy(p + SHMLOG_DATA, t.b, l);
+	vsl_hdr(tag, p, l, id);
 	w->wlr++;
+	if (params->diag_bitmap & 0x10000)
+		WSL_Flush(w, 0);
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-WSL(struct worker *w, enum shmlogtag tag, unsigned id, const char *fmt, ...)
+WSL(struct worker *w, enum shmlogtag tag, int id, const char *fmt, ...)
 {
 	va_list ap;
 	unsigned char *p;
-	unsigned n;
+	unsigned n, mlen;
+	txt t;
 
+	AN(fmt);
 	va_start(ap, fmt);
+	mlen = params->shm_reclen;
 
 	if (strchr(fmt, '%') == NULL) {
-		WSLR(w, tag, id, fmt, NULL);
-		return;
+		t.b = TRUST_ME(fmt);
+		t.e = strchr(t.b, '\0');
+		WSLR(w, tag, id, t);
+	} else {
+		assert(w->wlp < w->wle);
+
+		/* Wrap if we cannot fit a full size record */
+		if (w->wlp + SHMLOG_NEXTTAG + mlen + 1 >= w->wle)
+			WSL_Flush(w, 1);
+
+		p = w->wlp;
+		/* +1 for the NUL */
+		n = vsnprintf((char *)(p + SHMLOG_DATA), mlen + 1, fmt, ap);
+		if (n > mlen)
+			n = mlen;	/* we truncate long fields */
+		vsl_hdr(tag, p, n, id);
+		w->wlp += SHMLOG_NEXTTAG + n;
+		assert(w->wlp < w->wle);
+		w->wlr++;
 	}
-
-	assert(w->wlp < w->wle);
-
-	/* Wrap if we cannot fit a full size record */
-	if (w->wlp + 5 + 255 + 1 >= w->wle)
-		WSL_Flush(w);
-
-	p = w->wlp;
-	n = 0;
-	if (fmt != NULL) {
-		n = vsnprintf((char *)(p + 4), 256, fmt, ap);
-		if (n > 255)
-			n = 255; 	/* we truncate long fields */
-	}
-	p[1] = n & 0xff;
-	p[2] = (id >> 8) & 0xff;
-	p[3] = id & 0xff;
-	p[4 + n] = '\0';;
-	p[5 + n] = SLT_ENDMARKER;
-	p[0] = tag;
-
-	w->wlp += 5 + n;
-	assert(w->wlp < w->wle);
-	w->wlr++;
 	va_end(ap);
+	if (params->diag_bitmap & 0x10000)
+		WSL_Flush(w, 0);
+}
+
+/*--------------------------------------------------------------------*/
+
+void
+VSL_Panic(int *len, char **ptr)
+{
+
+	AN(len);
+	AN(ptr);
+	assert(loghead->magic == SHMLOGHEAD_MAGIC);
+	assert(loghead->hdrsize == sizeof *loghead);
+	*len = sizeof(loghead->panicstr);
+	*ptr = loghead->panicstr;
 }
 
 /*--------------------------------------------------------------------*/
@@ -288,8 +299,9 @@ VSL_Init(void)
 	assert(loghead->hdrsize == sizeof *loghead);
 	/* XXX more check sanity of loghead  ? */
 	logstart = (unsigned char *)loghead + loghead->start;
-	MTX_INIT(&vsl_mtx);
+	AZ(pthread_mutex_init(&vsl_mtx, NULL));
 	loghead->starttime = TIM_real();
+	loghead->panicstr[0] = '\0';
 	memset(VSL_stats, 0, sizeof *VSL_stats);
 }
 
@@ -363,6 +375,10 @@ VSL_MgtInit(const char *fn, unsigned size)
 	    MAP_HASSEMAPHORE | MAP_NOSYNC | MAP_SHARED,
 	    heritage.vsl_fd, 0);
 	xxxassert(loghead != MAP_FAILED);
+	i = mlock(loghead, heritage.vsl_size);
+	if (i != 0)
+		fprintf(stderr, "Notice: locking SHMFILE in core failed: %s\n",
+		    strerror(errno));
 	VSL_stats = &loghead->stats;
 	pp = (void *)(loghead + 1);
 	*pp = *params;
