@@ -1,8 +1,8 @@
 /*-
- * Copyright (c) 2007 Linpro AS
+ * Copyright (c) 2007-2008 Linpro AS
  * All rights reserved.
  *
- * Author: Dag-Erling Smrgav <des@linpro.no>
+ * Author: Dag-Erling Smørgav <des@des.no>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,29 +25,58 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: stevedore.c 1723 2007-07-19 12:20:28Z des $
+ * $Id: stevedore.c 3598 2009-02-05 10:10:58Z tfheen $
  */
 
+#include "config.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
 #include "cache.h"
+#include "stevedore.h"
+
+static VTAILQ_HEAD(, stevedore)	stevedores =
+    VTAILQ_HEAD_INITIALIZER(stevedores);
+
+static const struct stevedore * volatile stv_next;
 
 struct storage *
-STV_alloc(size_t size)
+STV_alloc(struct sess *sp, size_t size)
 {
 	struct storage *st;
+	struct stevedore *stv;
 
-	AN(stevedore);
-	AN(stevedore->alloc);
-	do {
-		if ((st = stevedore->alloc(stevedore, size)) == NULL)
-			AN(LRU_DiscardOne());
-	} while (st == NULL);
+	for (;;) {
+
+		/* pick a stevedore and bump the head along */
+		stv = VTAILQ_NEXT(stv_next, list);
+		if (stv == NULL)
+			stv = VTAILQ_FIRST(&stevedores);
+		AN(stv);
+
+		 /* XXX: only safe as long as pointer writes are atomic */
+		stv_next = stv;
+
+		/* try to allocate from it */
+		st = stv->alloc(stv, size);
+		if (st != NULL)
+			break;
+
+		/* no luck; try to free some space and keep trying */
+		if (EXP_NukeOne(sp) == -1)
+			break;
+	}
+	CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
 	return (st);
 }
 
 void
-STV_trim(struct storage *st, size_t size)
+STV_trim(const struct storage *st, size_t size)
 {
 
+	CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
 	AN(st->stevedore);
 	if (st->stevedore->trim)
 		st->stevedore->trim(st, size);
@@ -57,7 +86,41 @@ void
 STV_free(struct storage *st)
 {
 
+	CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
 	AN(st->stevedore);
-	AN(stevedore->free);
+	AN(st->stevedore->free);
 	st->stevedore->free(st);
 }
+
+void
+STV_add(const struct stevedore *stv2, int ac, char * const *av)
+{
+	struct stevedore *stv;
+
+	CHECK_OBJ_NOTNULL(stv2, STEVEDORE_MAGIC);
+	ALLOC_OBJ(stv, STEVEDORE_MAGIC);
+	AN(stv);
+
+	*stv = *stv2;
+
+	if (stv->init != NULL)
+		stv->init(stv, ac, av);
+	else if (ac != 0)
+		ARGV_ERR("(-s%s) too many arguments\n", stv->name);
+
+	VTAILQ_INSERT_TAIL(&stevedores, stv, list);
+
+	if (!stv_next)
+		stv_next = VTAILQ_FIRST(&stevedores);
+}
+
+void
+STV_open(void)
+{
+	struct stevedore *stv;
+
+	VTAILQ_FOREACH(stv, &stevedores, list) {
+		if (stv->open != NULL)
+			stv->open(stv);
+	}
+}
