@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: varnishlog.c 3625 2009-02-05 12:36:41Z tfheen $
+ * $Id: varnishlog.c 4003 2009-03-27 12:13:31Z tfheen $
  *
  * Log tailer for Varnish
  */
@@ -372,7 +372,7 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	if (D_flag && daemon(0, 0) == -1) {
+	if (D_flag && inxorcise(0, 0) == -1) {
 		perror("daemon()");
 		if (pfh != NULL)
 			vpf_remove(pfh);
