@@ -1,7 +1,9 @@
 /*
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2008-2009 Linpro AS
  * All rights reserved.
  *
+ * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -23,7 +25,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vtc.c 3586 2009-02-05 09:17:38Z tfheen $
+ * $Id: vtc.c 3999 2009-03-23 14:51:24Z tfheen $
  */
 
 #include "config.h"
@@ -34,6 +36,7 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <sys/wait.h>
 
 #include "libvarnish.h"
 #include "vsb.h"
@@ -235,16 +238,17 @@ cmd_test(CMD_ARGS)
 static void
 cmd_shell(CMD_ARGS)
 {
-
 	(void)priv;
 	(void)cmd;
+	int r;
 
 	if (av == NULL)
 		return;
 	AN(av[1]);
 	AZ(av[2]);
 	vtc_dump(vl, 4, "shell", av[1]);
-	(void)system(av[1]);	/* XXX: assert ? */
+	r = system(av[1]);
+	assert(WEXITSTATUS(r) == 0);
 }
 
 /**********************************************************************
@@ -264,11 +268,7 @@ cmd_delay(CMD_ARGS)
 	AZ(av[2]);
 	f = strtod(av[1], NULL);
 	vtc_log(vl, 3, "delaying %g second(s)", f);
-	if (f > 100.) {
-		(void)sleep((int)f);
-	} else {
-		(void)usleep((int)(f * 1e6));
-	}
+	TIM_sleep(f);
 }
 
 /**********************************************************************
