@@ -1,10 +1,10 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
- * Author: Dag-Erling Smørgrav <des@linpro.no>
+ * Author: Dag-Erling Smørgrav <des@des.no>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,11 +27,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: varnishtop.c 2034 2007-09-25 08:54:11Z des $
+ * $Id: varnishtop.c 3586 2009-02-05 09:17:38Z tfheen $
  *
  * Log tailer for Varnish
  */
 
+#include "config.h"
+
 #include <ctype.h>
 #include <curses.h>
 #include <errno.h>
@@ -43,11 +45,12 @@
 #include <unistd.h>
 #include <limits.h>
 
+#include "vqueue.h"
+
 #include "vsb.h"
 
 #include "libvarnish.h"
 #include "shmlog.h"
-#include "vqueue.h"
 #include "varnishapi.h"
 
 struct top {
@@ -68,19 +71,22 @@ static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
 
 static int f_flag = 0;
 
+static unsigned maxfieldlen = 0;
+
 static void
 accumulate(const unsigned char *p)
 {
 	struct top *tp, *tp2;
 	const unsigned char *q;
-	unsigned int u;
+	unsigned int u, l;
 	int i;
 
 	// fprintf(stderr, "%*.*s\n", p[1], p[1], p + 4);
 
 	u = 0;
-	q = p + 4;
-	for (i = 0; i < p[1]; i++, q++) {
+	q = p + SHMLOG_DATA;
+	l = SHMLOG_LEN(p);
+	for (i = 0; i < l; i++, q++) {
 		if (f_flag && (*q == ':' || isspace(*q)))
 			break;
 		u += *q;
@@ -89,11 +95,12 @@ accumulate(const unsigned char *p)
 	VTAILQ_FOREACH(tp, &top_head, list) {
 		if (tp->hash != u)
 			continue;
-		if (tp->rec[0] != p[0])
+		if (tp->rec[SHMLOG_TAG] != p[SHMLOG_TAG])
 			continue;
 		if (tp->clen != q - p)
 			continue;
-		if (memcmp(p + 4, tp->rec + 4, q - (p + 4)))
+		if (memcmp(p + SHMLOG_DATA, tp->rec + SHMLOG_DATA,
+		    q - (p + SHMLOG_DATA)))
 			continue;
 		tp->count += 1.0;
 		break;
@@ -107,7 +114,7 @@ accumulate(const unsigned char *p)
 		tp->clen = q - p;
 		VTAILQ_INSERT_TAIL(&top_head, tp, list);
 	}
-	memcpy(tp->rec, p, 4 + p[1]);
+	memcpy(tp->rec, p, SHMLOG_DATA + l);
 	while (1) {
 		tp2 = VTAILQ_PREV(tp, tophead, list);
 		if (tp2 == NULL || tp2->count >= tp->count)
@@ -128,7 +135,7 @@ static void
 update(void)
 {
 	struct top *tp, *tp2;
-	int l;
+	int l, len;
 	double t = 0;
 	static time_t last;
 	time_t now;
@@ -144,12 +151,13 @@ update(void)
 	mvprintw(0, 0, "list length %u", ntop);
 	VTAILQ_FOREACH_SAFE(tp, &top_head, list, tp2) {
 		if (++l < LINES) {
-			int len = tp->rec[1];
+			len = SHMLOG_LEN(tp->rec);
 			if (len > COLS - 20)
 				len = COLS - 20;
-			mvprintw(l, 0, "%9.2f %-9.9s %*.*s\n",
-			    tp->count, VSL_tags[tp->rec[0]],
-			    len, len, tp->rec + 4);
+			mvprintw(l, 0, "%9.2f %-*.*s %*.*s\n",
+			    tp->count, maxfieldlen, maxfieldlen,
+			    VSL_tags[tp->rec[SHMLOG_TAG]],
+			    len, len, tp->rec + SHMLOG_DATA);
 			t = tp->count;
 		}
 		tp->count *= .999;
@@ -191,6 +199,14 @@ do_curses(struct VSL_data *vd)
 {
 	pthread_t thr;
 	int ch;
+	int i;
+
+	for (i = 0; i < 256; i++) {
+		if (VSL_tags[i] == NULL)
+			continue;
+		if (maxfieldlen < strlen(VSL_tags[i]))
+			maxfieldlen = strlen(VSL_tags[i]);
+	}
 
 	if (pthread_create(&thr, NULL, accumulate_thread, vd) != 0) {
 		fprintf(stderr, "pthread_create(): %s\n", strerror(errno));
@@ -213,9 +229,11 @@ do_curses(struct VSL_data *vd)
 		switch ((ch = getch())) {
 		case ERR:
 			break;
+#ifdef KEY_RESIZE
 		case KEY_RESIZE:
 			erase();
 			break;
+#endif
 		case '\014': /* Ctrl-L */
 		case '\024': /* Ctrl-T */
 			redrawwin(stdscr);
@@ -267,7 +285,8 @@ do_once(struct VSL_data *vd)
 static void
 usage(void)
 {
-	fprintf(stderr, "usage: varnishtop %s [-1fV] [-n varnish_name]\n", VSL_USAGE);
+	fprintf(stderr,
+	    "usage: varnishtop %s [-1fV] [-n varnish_name]\n", VSL_USAGE);
 	exit(1);
 }
 
