@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cli.h 1693 2007-07-13 14:54:51Z des $
+ * $Id: cli.h 3699 2009-02-09 12:24:17Z tfheen $
  *
  * Public definition of the CLI protocol, part of the published Varnish-API.
  *
@@ -57,16 +57,36 @@
 	"url.query",							\
 	"url.query <url>",						\
 	"\tQuery the cache status of a specific URL.\n"			\
-	    "\tReturns the TTL, size and checksum of the object.", 	\
+	    "\tReturns the TTL, size and checksum of the object.",	\
 	1, 1
 
-#define CLI_URL_PURGE							\
-	"url.purge",							\
-	"url.purge <regexp>",						\
-	"\tAll urls matching regexp will consider currently cached\n"	\
-	    "\tobjects obsolete",					\
+#define CLI_PURGE_URL							\
+	"purge.url",							\
+	"purge.url <regexp>",						\
+	"\tAll objects where the urls matches regexp will be "		\
+	    "marked obsolete.",						\
 	1, 1
 
+#define CLI_PURGE_HASH							\
+	"purge.hash",							\
+	"purge.hash <regexp>",						\
+	"\tAll objects where the hash string matches regexp will be "	\
+	    "marked obsolete.",						\
+	1, 1
+
+#define CLI_PURGE							\
+	"purge",							\
+	"purge <field> <operator> <arg> [&& <field> <oper> <arg>]...",	\
+	"\tAll objects where the all the conditions match will be "	\
+	    "marked obsolete.",						\
+	3, UINT_MAX 
+
+#define CLI_PURGE_LIST							\
+	"purge.list",							\
+	"purge.list",							\
+	"\tList the active purges.",					\
+	0, 0
+
 #define CLI_URL_STATUS							\
 	"url.status",							\
 	"url.status <url>",						\
@@ -193,30 +213,21 @@
 	"\tEnable/Disable verbosity",					\
 	0, 0
 
-#define CLI_EXIT							\
-	"exit",								\
-	"exit",								\
-	"\tClose connection",						\
-	0, 0
-
 #define CLI_QUIT							\
 	"quit",								\
 	"quit",								\
 	"\tClose connection",						\
 	0, 0
 
-#define CLI_BYE								\
-	"bye",								\
-	"bye",								\
-	"\tClose connection",						\
-	0, 0
-
-# define CLI_SERVER_STATUS						\
+#define CLI_SERVER_STATUS						\
 	"status",							\
 	"status",							\
 	"\tCheck status of Varnish cache process.",			\
 	0, 0
 
+#define CLI_HIDDEN(foo, min_arg, max_arg)				\
+	foo, NULL, NULL, min_arg, max_arg,
+
 /*
  * Status/return codes in the CLI protocol
  */
@@ -230,7 +241,8 @@ enum cli_status_e {
 	CLIS_PARAM	= 106,
 	CLIS_OK		= 200,
 	CLIS_CANT	= 300,
-	CLIS_COMMS	= 400
+	CLIS_COMMS	= 400,
+	CLIS_CLOSE	= 500
 };
 
 /* Length of first line of response */
