@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: libvarnish.h 3670 2009-02-06 13:41:00Z tfheen $
+ * $Id: libvarnish.h 4003 2009-03-27 12:13:31Z tfheen $
  */
 
 #include <errno.h>
@@ -71,7 +71,7 @@ void TCP_name(const struct sockaddr *addr, unsigned l, char *abuf,
 int TCP_connect(int s, const struct sockaddr *name, socklen_t namelen,
     int msec);
 void TCP_close(int *s);
-void TCP_set_read_timeout(int socket, double seconds);
+void TCP_set_read_timeout(int s, double seconds);
 #endif
 
 /* from libvarnish/time.c */
@@ -130,3 +130,5 @@ do {									\
 	lbv_assert(__func__, __FILE__, __LINE__, expl, errno, 3);	\
 	abort();							\
 } while (0)
+
+int inxorcise(int nochdir, int noclose);
