@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: shmlog.h 1521 2007-06-15 09:18:06Z cecilihf $
+ * $Id: shmlog.h 2971 2008-07-20 10:36:28Z phk $
  *
  * Define the layout of the shared memory log segment.
  *
@@ -63,17 +63,31 @@ struct shmloghead {
 	unsigned		ptr;
 
 	struct varnish_stats	stats;
+
+	/* Panic message buffer */
+	char			panicstr[64 * 1024];
 };
 
 /*
  * Record format is as follows:
  *
  *	1 byte		field type (enum shmlogtag)
- *	1 byte		length of contents
- *	2 byte		record identifier
+ *	2 bytes		length of contents
+ *	2 bytes		record identifier
  *	n bytes		field contents (isgraph(c) || isspace(c)) allowed.
  */
 
+#define SHMLOG_TAG	0
+#define __SHMLOG_LEN_HIGH	1
+#define __SHMLOG_LEN_LOW	2
+#define __SHMLOG_ID_HIGH	3
+#define __SHMLOG_ID_LOW	4
+#define SHMLOG_DATA	5
+#define SHMLOG_NEXTTAG	6	/* ... + len */
+
+#define SHMLOG_LEN(p)	(((p)[__SHMLOG_LEN_HIGH] << 8) | (p)[__SHMLOG_LEN_LOW])
+#define SHMLOG_ID(p)	(((p)[__SHMLOG_ID_HIGH] << 8) | (p)[__SHMLOG_ID_LOW])
+
 /*
  * The identifiers in shmlogtag are "SLT_" + XML tag.  A script may be run
  * on this file to extract the table rather than handcode it
