@@ -11,8 +11,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -25,7 +23,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: vsb.h 1818 2007-08-09 12:12:12Z des $
+ * $Id: vsb.h 3704 2009-02-09 12:43:02Z tfheen $
  * $FreeBSD: src/sys/sys/vsb.h,v 1.14 2004/07/09 11:35:30 des Exp $
  */
 
@@ -36,6 +34,7 @@
  * Structure definition
  */
 struct vsb {
+	unsigned	s_magic;
 	char		*s_buf;		/* storage buffer */
 	void		*s_unused;	/* binary compatibility. */
 	int		 s_size;	/* size of storage buffer */
@@ -57,24 +56,28 @@ extern "C" {
  * API functions
  */
 struct vsb	*vsb_new(struct vsb *, char *, int, int);
+#define vsb_newauto()	vsb_new(NULL, NULL, 0, VSB_AUTOEXTEND)
 void		 vsb_clear(struct vsb *);
 int		 vsb_setpos(struct vsb *, int);
 int		 vsb_bcat(struct vsb *, const void *, size_t);
 int		 vsb_bcpy(struct vsb *, const void *, size_t);
 int		 vsb_cat(struct vsb *, const char *);
 int		 vsb_cpy(struct vsb *, const char *);
-int		 vsb_printf(struct vsb *, const char *, ...) /* __printflike(2, 3) */;
+int		 vsb_printf(struct vsb *, const char *, ...)
+    /* __printflike(2, 3) */;
 #ifdef va_start
-int		 vsb_vprintf(struct vsb *, const char *, va_list) /* __printflike(2, 0) */;
+int		 vsb_vprintf(struct vsb *, const char *, va_list)
+    /* __printflike(2, 0) */;
 #endif
 int		 vsb_putc(struct vsb *, int);
 int		 vsb_trim(struct vsb *);
-int		 vsb_overflowed(struct vsb *);
+int		 vsb_overflowed(const struct vsb *);
 void		 vsb_finish(struct vsb *);
 char		*vsb_data(struct vsb *);
 int		 vsb_len(struct vsb *);
-int		 vsb_done(struct vsb *);
+int		 vsb_done(const struct vsb *);
 void		 vsb_delete(struct vsb *);
+void		 vsb_quote(struct vsb *s, const char *p, int how);
 #ifdef __cplusplus
 };
 #endif
