@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,34 +26,39 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: assert.c 2146 2007-10-19 11:36:35Z des $
+ * $Id: assert.c 3586 2009-02-05 09:17:38Z tfheen $
+ *
+ * This is the default backend function for libvarnish' assert facilities.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
-#include <errno.h>
 #include <string.h>
 
 #include "libvarnish.h"
 
-void
-lbv_xxxassert(const char *func, const char *file, int line, const char *cond, int err)
+static void
+lbv_assert_default(const char *func, const char *file, int line,
+    const char *cond, int err, int xxx)
 {
 
-	fprintf(stderr,
-	    "Missing errorhandling code in %s(), %s line %d:\n"
-	    "  Condition(%s) not true.\n"
-	    "  errno = %d (%s)\n", func, file, line, cond, err, strerror(err));
+	if (xxx) {
+		fprintf(stderr,
+		    "Missing errorhandling code in %s(), %s line %d:\n"
+		    "  Condition(%s) not true.\n",
+		    func, file, line, cond);
+	} else {
+		fprintf(stderr,
+		    "Assert error in %s(), %s line %d:\n"
+		    "  Condition(%s) not true.\n",
+		    func, file, line, cond);
+	}
+	if (err)
+		fprintf(stderr,
+		    "  errno = %d (%s)\n", err, strerror(err));
 	abort();
 }
 
-void
-lbv_assert(const char *func, const char *file, int line, const char *cond, int err)
-{
-
-	fprintf(stderr,
-	    "Assert error in %s(), %s line %d:\n"
-	    "  Condition(%s) not true.\n"
-	    "  errno = %d (%s)\n", func, file, line, cond, err, strerror(err));
-	abort();
-}
+lbv_assert_f *lbv_assert = lbv_assert_default;
