@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: time.c 3586 2009-02-05 09:17:38Z tfheen $
+ * $Id: time.c 3998 2009-03-23 14:45:32Z tfheen $
  *
  * Semi-trivial functions to handle HTTP header timestamps according to
  * RFC 2616 section 3.3.
@@ -152,10 +152,20 @@ TIM_parse(const char *p)
 void
 TIM_sleep(double t)
 {
-	if (t > 100.0)
-		(void)sleep((int)round(t));
-	else
-		(void)usleep((int)round(t * 1e6));
+	struct timespec ts;
+
+	ts.tv_sec = floor(t);
+	ts.tv_nsec = floor((t - ts.tv_sec) * 1e9);
+
+#ifdef HAVE_NANOSLEEP
+	(void)nanosleep(&ts, NULL);
+#else
+	if (ts.tv_sec > 0)
+		(void)sleep(ts.tv_sec);
+	ts.tv_nsec /= 1000;
+	if (ts.tv_nsec > 0)
+		(void)usleep(ts.tv_nsec);
+#endif
 }
 
 
