@@ -23,11 +23,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vpf.c 1818 2007-08-09 12:12:12Z des $
+ * $Id: vpf.c 3586 2009-02-05 09:17:38Z tfheen $
  * Derived from:
- * $FreeBSD: src/lib/libutil/pidfile.c,v 1.5 2007/05/11 11:10:05 des Exp $
+ * $FreeBSD: head/lib/libutil/pidfile.c 184091 2008-10-20 17:41:08Z des $
  */
 
+#include "config.h"
+
 #include <sys/param.h>
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -43,6 +45,7 @@
 #include "compat/strlcpy.h"
 #endif
 
+#include "libvarnish.h"		/* XXX: for assert() */
 #include "flopen.h"
 #include "vpf.h"
 
@@ -56,7 +59,7 @@ struct pidfh {
 static int _vpf_remove(struct pidfh *pfh, int freeit);
 
 static int
-vpf_verify(struct pidfh *pfh)
+vpf_verify(const struct pidfh *pfh)
 {
 	struct stat sb;
 
@@ -84,7 +87,7 @@ vpf_read(const char *path, pid_t *pidptr)
 
 	i = read(fd, buf, sizeof(buf) - 1);
 	error = errno;	/* Remember errno in case close() wants to change it. */
-	close(fd);
+	(void)close(fd);
 	if (i == -1)
 		return (error);
 	buf[i] = '\0';
@@ -113,8 +116,11 @@ vpf_open(const char *path, mode_t mode, pid_t *pidptr)
 		    "/var/run/%s.pid", getprogname());
 	else
 #endif
+	{
+		assert(path != NULL);
 		len = snprintf(pfh->pf_path, sizeof(pfh->pf_path),
 		    "%s", path);
+	}
 	if (len >= (int)sizeof(pfh->pf_path)) {
 		free(pfh);
 		errno = ENAMETOOLONG;
@@ -144,8 +150,8 @@ vpf_open(const char *path, mode_t mode, pid_t *pidptr)
 	 */
 	if (fstat(fd, &sb) == -1) {
 		error = errno;
-		unlink(pfh->pf_path);
-		close(fd);
+		(void)unlink(pfh->pf_path);
+		(void)close(fd);
 		free(pfh);
 		errno = error;
 		return (NULL);
@@ -182,15 +188,16 @@ vpf_write(struct pidfh *pfh)
 	 */
 	if (ftruncate(fd, 0) == -1) {
 		error = errno;
-		_vpf_remove(pfh, 0);
+		(void)_vpf_remove(pfh, 0);
 		errno = error;
 		return (-1);
 	}
 
-	snprintf(pidstr, sizeof(pidstr), "%u", getpid());
+	error = snprintf(pidstr, sizeof(pidstr), "%u", getpid());
+	assert(error < sizeof pidstr);
 	if (pwrite(fd, pidstr, strlen(pidstr), 0) != (ssize_t)strlen(pidstr)) {
 		error = errno;
-		_vpf_remove(pfh, 0);
+		(void)_vpf_remove(pfh, 0);
 		errno = error;
 		return (-1);
 	}
@@ -222,14 +229,8 @@ vpf_close(struct pidfh *pfh)
 static int
 _vpf_remove(struct pidfh *pfh, int freeit)
 {
-	struct flock lock;
 	int error;
 
-	lock.l_type = F_UNLCK;
-	lock.l_start = 0;
-	lock.l_whence = SEEK_SET;
-	lock.l_len = 0;
-
 	error = vpf_verify(pfh);
 	if (error != 0) {
 		errno = error;
@@ -238,10 +239,6 @@ _vpf_remove(struct pidfh *pfh, int freeit)
 
 	if (unlink(pfh->pf_path) == -1)
 		error = errno;
-	if (fcntl(pfh->pf_fd, F_SETLK, &lock) == -1) {
-		if (error == 0)
-			error = errno;
-	}
 	if (close(pfh->pf_fd) == -1) {
 		if (error == 0)
 			error = errno;
