@@ -11,8 +11,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -25,24 +23,27 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: vsb.c 2239 2007-11-08 21:30:05Z des $
+ * $Id: vsb.c 3704 2009-02-09 12:43:02Z tfheen $
  * $FreeBSD: src/sys/kern/subr_sbuf.c,v 1.30 2005/12/23 11:49:53 phk Exp $
  */
 
+#include "config.h"
+
 #include <ctype.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strings.h>
 
+#include "libvarnish.h"
 #include "vsb.h"
 
-#define	KASSERT(e, m)
+#define	KASSERT(e, m)		assert(e)
 #define	SBMALLOC(size)		malloc(size)
 #define	SBFREE(buf)		free(buf)
 #define	min(x,y)		(x < y ? x : y)
 
+#define VSB_MAGIC		0x4a82dd8a
 /*
  * Predicates
  */
@@ -51,7 +52,7 @@
 #define	VSB_ISFINISHED(s)	((s)->s_flags & VSB_FINISHED)
 #define	VSB_HASOVERFLOWED(s)	((s)->s_flags & VSB_OVERFLOWED)
 #define	VSB_HASROOM(s)		((s)->s_len < (s)->s_size - 1)
-#define	VSB_FREESPACE(s)	((s)->s_size - (s)->s_len - 1)
+#define	VSB_FREESPACE(s)	((s)->s_size - ((s)->s_len + 1))
 #define	VSB_CANEXTEND(s)	((s)->s_flags & VSB_AUTOEXTEND)
 
 /*
@@ -75,6 +76,8 @@ _vsb_assert_integrity(const char *fun, struct vsb *s)
 	(void)s;
 	KASSERT(s != NULL,
 	    ("%s called with a NULL vsb pointer", fun));
+	KASSERT(s->s_magic == VSB_MAGIC,
+	    ("%s called wih an unintialized vsb pointer", fun));
 	KASSERT(s->s_buf != NULL,
 	    ("%s called with uninitialized or corrupt vsb", fun));
 	KASSERT(s->s_len < s->s_size,
@@ -131,7 +134,7 @@ vsb_extend(struct vsb *s, int addlen)
 	newbuf = (char *)SBMALLOC(newsize);
 	if (newbuf == NULL)
 		return (-1);
-	bcopy(s->s_buf, newbuf, s->s_size);
+	memcpy(newbuf, s->s_buf, s->s_size);
 	if (VSB_ISDYNAMIC(s))
 		SBFREE(s->s_buf);
 	else
@@ -159,13 +162,17 @@ vsb_new(struct vsb *s, char *buf, int length, int flags)
 		s = (struct vsb *)SBMALLOC(sizeof *s);
 		if (s == NULL)
 			return (NULL);
-		bzero(s, sizeof *s);
-		s->s_flags = flags;
+		if (vsb_new(s, buf, length, flags) == NULL) {
+			free(s);
+			return (NULL);
+		}
 		VSB_SETFLAG(s, VSB_DYNSTRUCT);
-	} else {
-		bzero(s, sizeof *s);
-		s->s_flags = flags;
+		return (s);
 	}
+
+	memset(s, 0, sizeof *s);
+	s->s_flags = flags;
+	s->s_magic = VSB_MAGIC;
 	s->s_size = length;
 	if (buf) {
 		s->s_buf = buf;
@@ -174,11 +181,8 @@ vsb_new(struct vsb *s, char *buf, int length, int flags)
 	if (flags & VSB_AUTOEXTEND)
 		s->s_size = vsb_extendsize(s->s_size);
 	s->s_buf = (char *)SBMALLOC(s->s_size);
-	if (s->s_buf == NULL) {
-		if (VSB_ISDYNSTRUCT(s))
-			SBFREE(s);
+	if (s->s_buf == NULL)
 		return (NULL);
-	}
 	VSB_SETFLAG(s, VSB_DYNAMIC);
 	return (s);
 }
@@ -373,7 +377,7 @@ vsb_putc(struct vsb *s, int c)
 		return (-1);
 	}
 	if (c != '\0')
-	    s->s_buf[s->s_len++] = c;
+	    s->s_buf[s->s_len++] = (char)c;
 	return (0);
 }
 
@@ -399,7 +403,7 @@ vsb_trim(struct vsb *s)
  * Check if an vsb overflowed
  */
 int
-vsb_overflowed(struct vsb *s)
+vsb_overflowed(const struct vsb *s)
 {
     return VSB_HASOVERFLOWED(s);
 }
@@ -458,7 +462,7 @@ vsb_delete(struct vsb *s)
 	if (VSB_ISDYNAMIC(s))
 		SBFREE(s->s_buf);
 	isdyn = VSB_ISDYNSTRUCT(s);
-	bzero(s, sizeof *s);
+	memset(s, 0, sizeof *s);
 	if (isdyn)
 		SBFREE(s);
 }
@@ -467,8 +471,60 @@ vsb_delete(struct vsb *s)
  * Check if an vsb has been finished.
  */
 int
-vsb_done(struct vsb *s)
+vsb_done(const struct vsb *s)
 {
 
 	return(VSB_ISFINISHED(s));
 }
+
+/*
+ * Quote a string
+ */
+void
+vsb_quote(struct vsb *s, const char *p, int how)
+{
+	const char *q;
+	int quote = 0;
+
+	(void)how;	/* For future enhancements */
+
+	for (q = p; *q != '\0'; q++) {
+		if (!isgraph(*q) || *q == '"') {
+			quote++;
+			break;
+		}
+	}
+	if (!quote) {
+		(void)vsb_cat(s, p);
+		return;
+	}
+	(void)vsb_putc(s, '"');
+	for (q = p; *q != '\0'; q++) {
+		switch (*q) {
+		case ' ':
+			(void)vsb_putc(s, *q);
+			break;
+		case '\\':
+		case '"':
+			(void)vsb_putc(s, '\\');
+			(void)vsb_putc(s, *q);
+			break;
+		case '\n':
+			(void)vsb_cat(s, "\\n");
+			break;
+		case '\r':
+			(void)vsb_cat(s, "\\r");
+			break;
+		case '\t':
+			(void)vsb_cat(s, "\\t");
+			break;
+		default:
+			if (isgraph(*q))
+				(void)vsb_putc(s, *q);
+			else
+				(void)vsb_printf(s, "\\%o", *q);
+			break;
+		}
+	}
+	(void)vsb_putc(s, '"');
+}
