@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,9 +26,12 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: shmlog.c 1588 2007-06-28 10:25:47Z des $
+ * $Id: shmlog.c 3586 2009-02-05 09:17:38Z tfheen $
  */
 
+#include "config.h"
+
+#include <sys/types.h>
 #include <sys/mman.h>
 
 #include <assert.h>
@@ -62,8 +65,8 @@ struct VSL_data {
 	unsigned char		*ptr;
 
 	/* for -r option */
-	FILE			*fi;
-	unsigned char		rbuf[5 + 255 + 1];
+	int			fd;
+	unsigned char		rbuf[SHMLOG_NEXTTAG + 255 + 1];
 
 	int			b_opt;
 	int			c_opt;
@@ -82,6 +85,9 @@ struct VSL_data {
 	int			regflags;
 	regex_t			*regincl;
 	regex_t			*regexcl;
+
+	unsigned long		skip;
+	unsigned long		keep;
 };
 
 #ifndef MAP_HASSEMAPHORE
@@ -165,6 +171,7 @@ VSL_New(void)
 	assert(vd != NULL);
 	vd->regflags = REG_EXTENDED | REG_NOSUB;
 	vd->magic = VSL_MAGIC;
+	vd->fd = -1;
 	return (vd);
 }
 
@@ -186,7 +193,7 @@ VSL_OpenLog(struct VSL_data *vd, const char *varnish_name)
 	unsigned char *p;
 
 	CHECK_OBJ_NOTNULL(vd, VSL_MAGIC);
-	if (vd->fi != NULL)
+	if (vd->fd != -1)
 		return (0);
 
 	if (vsl_shmem_map(varnish_name))
@@ -197,9 +204,9 @@ VSL_OpenLog(struct VSL_data *vd, const char *varnish_name)
 	vd->logend = vd->logstart + vsl_lh->size;
 	vd->ptr = vd->logstart;
 
-	if (!vd->d_opt && vd->fi == NULL) {
+	if (!vd->d_opt && vd->fd == -1) {
 		for (p = vd->ptr; *p != SLT_ENDMARKER; )
-			p += p[1] + 5;
+			p += SHMLOG_LEN(p) + SHMLOG_NEXTTAG;
 		vd->ptr = p;
 	}
 	return (0);
@@ -222,16 +229,17 @@ static int
 vsl_nextlog(struct VSL_data *vd, unsigned char **pp)
 {
 	unsigned char *p;
-	unsigned w;
+	unsigned w, l;
 	int i;
 
 	CHECK_OBJ_NOTNULL(vd, VSL_MAGIC);
-	if (vd->fi != NULL) {
-		i = fread(vd->rbuf, 4, 1, vd->fi);
-		if (i != 1)
+	if (vd->fd != -1) {
+		i = read(vd->fd, vd->rbuf, SHMLOG_DATA);
+		if (i != SHMLOG_DATA)
 			return (-1);
-		i = fread(vd->rbuf + 4, vd->rbuf[1] + 1, 1, vd->fi);
-		if (i != 1)
+		i = read(vd->fd, vd->rbuf + SHMLOG_DATA,
+		    SHMLOG_LEN(vd->rbuf) + 1);
+		if (i != SHMLOG_LEN(vd->rbuf) + 1)
 			return (-1);
 		*pp = vd->rbuf;
 		return (1);
@@ -250,7 +258,8 @@ vsl_nextlog(struct VSL_data *vd, unsigned char **pp)
 			usleep(SLEEP_USEC);
 			continue;
 		}
-		vd->ptr = p + p[1] + 5;
+		l = SHMLOG_LEN(p);
+		vd->ptr = p + l + SHMLOG_NEXTTAG;
 		*pp = p;
 		return (1);
 	}
@@ -263,7 +272,7 @@ VSL_NextLog(struct VSL_data *vd, unsigned char **pp)
 {
 	unsigned char *p;
 	regmatch_t rm;
-	unsigned u;
+	unsigned u, l;
 	int i;
 
 	CHECK_OBJ_NOTNULL(vd, VSL_MAGIC);
@@ -271,8 +280,9 @@ VSL_NextLog(struct VSL_data *vd, unsigned char **pp)
 		i = vsl_nextlog(vd, &p);
 		if (i != 1)
 			return (i);
-		u = (p[2] << 8) | p[3];
-		switch(p[0]) {
+		u = SHMLOG_ID(p);
+		l = SHMLOG_LEN(p);
+		switch(p[SHMLOG_TAG]) {
 		case SLT_SessionOpen:
 		case SLT_ReqStart:
 			vd->map[u] |= M_CLIENT;
@@ -286,11 +296,18 @@ VSL_NextLog(struct VSL_data *vd, unsigned char **pp)
 		default:
 			break;
 		}
-		if (vd->map[p[0]] & M_SELECT) {
+		if (vd->skip) {
+			--vd->skip;
+			continue;
+		} else if (vd->keep) {
+			if (--vd->keep == 0)
+				return (-1);
+		}
+		if (vd->map[p[SHMLOG_TAG]] & M_SELECT) {
 			*pp = p;
 			return (1);
 		}
-		if (vd->map[p[0]] & M_SUPPRESS)
+		if (vd->map[p[SHMLOG_TAG]] & M_SUPPRESS)
 			continue;
 		if (vd->b_opt && !(vd->map[u] & M_BACKEND))
 			continue;
@@ -298,15 +315,17 @@ VSL_NextLog(struct VSL_data *vd, unsigned char **pp)
 			continue;
 		if (vd->regincl != NULL) {
 			rm.rm_so = 0;
-			rm.rm_eo = p[1];
-			i = regexec(vd->regincl, (char *)p + 4, 1, &rm, 0);
+			rm.rm_eo = l;
+			i = regexec(vd->regincl,
+			    (char *)p + SHMLOG_DATA, 1, &rm, 0);
 			if (i == REG_NOMATCH)
 				continue;
 		}
 		if (vd->regexcl != NULL) {
 			rm.rm_so = 0;
-			rm.rm_eo = p[1];
-			i = regexec(vd->regexcl, (char *)p + 4, 1, &rm, 0);
+			rm.rm_eo = l;
+			i = regexec(vd->regexcl,
+			    (char *)p + SHMLOG_DATA, 1, &rm, 0);
 			if (i != REG_NOMATCH)
 				continue;
 		}
@@ -321,7 +340,7 @@ int
 VSL_Dispatch(struct VSL_data *vd, vsl_handler *func, void *priv)
 {
 	int i;
-	unsigned u;
+	unsigned u, l;
 	unsigned char *p;
 
 	CHECK_OBJ_NOTNULL(vd, VSL_MAGIC);
@@ -329,11 +348,12 @@ VSL_Dispatch(struct VSL_data *vd, vsl_handler *func, void *priv)
 		i = VSL_NextLog(vd, &p);
 		if (i <= 0)
 			return (i);
-		u = (p[2] << 8) | p[3];
+		u = SHMLOG_ID(p);
+		l = SHMLOG_LEN(p);
 		if (func(priv,
-		    p[0], u, p[1],
+		    p[SHMLOG_TAG], u, l,
 		    vd->map[u] & (VSL_S_CLIENT|VSL_S_BACKEND),
-		    (char *)p + 4))
+		    (char *)p + SHMLOG_DATA))
 			return (1);
 	}
 }
@@ -341,28 +361,30 @@ VSL_Dispatch(struct VSL_data *vd, vsl_handler *func, void *priv)
 /*--------------------------------------------------------------------*/
 
 int
-VSL_H_Print(void *priv, enum shmlogtag tag, unsigned fd, unsigned len, unsigned spec, const char *ptr)
+VSL_H_Print(void *priv, enum shmlogtag tag, unsigned fd, unsigned len,
+    unsigned spec, const char *ptr)
 {
 	FILE *fo = priv;
+	int type;
 
 	assert(fo != NULL);
+
+	type = (spec & VSL_S_CLIENT) ? 'c' :
+	    (spec & VSL_S_BACKEND) ? 'b' : '-';
+
 	if (tag == SLT_Debug) {
-		fprintf(fo, "%5d %-12s %c \"", fd, VSL_tags[tag],
-		    ((spec & VSL_S_CLIENT) ? 'c' : (spec & VSL_S_BACKEND) ? 'b' : ' '));
+		fprintf(fo, "%5d %-12s %c \"", fd, VSL_tags[tag], type);
 		while (len-- > 0) {
 			if (*ptr >= ' ' && *ptr <= '~')
 				fprintf(fo, "%c", *ptr);
 			else
-				fprintf(fo, "%%%02x", *ptr);
+				fprintf(fo, "%%%02x", (unsigned char)*ptr);
 			ptr++;
 		}
 		fprintf(fo, "\"\n");
 		return (0);
 	}
-	fprintf(fo, "%5d %-12s %c %.*s\n",
-	    fd, VSL_tags[tag],
-	    ((spec & VSL_S_CLIENT) ? 'c' : (spec & VSL_S_BACKEND) ? 'b' : ' '),
-	    len, ptr);
+	fprintf(fo, "%5d %-12s %c %.*s\n", fd, VSL_tags[tag], type, len, ptr);
 	return (0);
 }
 
@@ -374,13 +396,14 @@ vsl_r_arg(struct VSL_data *vd, const char *opt)
 
 	CHECK_OBJ_NOTNULL(vd, VSL_MAGIC);
 	if (!strcmp(opt, "-"))
-		vd->fi = stdin;
+		vd->fd = STDIN_FILENO;
 	else
-		vd->fi = fopen(opt, "r");
-	if (vd->fi != NULL)
-		return (1);
-	perror(opt);
-	return (-1);
+		vd->fd = open(opt, O_RDONLY);
+	if (vd->fd < 0) {
+		perror(opt);
+		return (-1);
+	}
+	return (1);
 }
 
 /*--------------------------------------------------------------------*/
@@ -469,6 +492,45 @@ vsl_ix_arg(struct VSL_data *vd, const char *opt, int arg)
 
 /*--------------------------------------------------------------------*/
 
+static int
+vsl_s_arg(struct VSL_data *vd, const char *opt)
+{
+	char *end;
+
+	CHECK_OBJ_NOTNULL(vd, VSL_MAGIC);
+	if (*opt == '\0') {
+		fprintf(stderr, "number required for -s\n");
+		return (-1);
+	}
+	vd->skip = strtoul(opt, &end, 10);
+	if (*end != '\0') {
+		fprintf(stderr, "invalid number for -s\n");
+		return (-1);
+	}
+	return (1);
+}
+/*--------------------------------------------------------------------*/
+
+static int
+vsl_k_arg(struct VSL_data *vd, const char *opt)
+{
+	char *end;
+
+	CHECK_OBJ_NOTNULL(vd, VSL_MAGIC);
+	if (*opt == '\0') {
+		fprintf(stderr, "number required for -k\n");
+		return (-1);
+	}
+	vd->keep = strtoul(opt, &end, 10);
+	if (*end != '\0') {
+		fprintf(stderr, "invalid number for -k\n");
+		return (-1);
+	}
+	return (1);
+}
+
+/*--------------------------------------------------------------------*/
+
 int
 VSL_Arg(struct VSL_data *vd, int arg, const char *opt)
 {
@@ -482,6 +544,8 @@ VSL_Arg(struct VSL_data *vd, int arg, const char *opt)
 	case 'r': return (vsl_r_arg(vd, opt));
 	case 'I': case 'X': return (vsl_IX_arg(vd, opt, arg));
 	case 'C': vd->regflags = REG_ICASE; return (1);
+	case 's': return (vsl_s_arg(vd, opt));
+	case 'k': return (vsl_k_arg(vd, opt));
 	default:
 		return (0);
 	}
