@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,14 +26,17 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vcc_compile.h 2034 2007-09-25 08:54:11Z des $
+ * $Id: vcc_compile.h 3676 2009-02-06 14:18:21Z tfheen $
  */
 
 #include "vqueue.h"
-#include "vcl_returns.h"
+
+#include "vcl.h"
 
 #define INDENT		2
 
+struct acl_e;
+
 struct membit {
 	VTAILQ_ENTRY(membit)	list;
 	void			*ptr;
@@ -64,6 +67,7 @@ struct tokenlist {
 	struct tokenhead	tokens;
 	VTAILQ_HEAD(, source)	sources;
 	VTAILQ_HEAD(, membit)	membits;
+	VTAILQ_HEAD(, host)	hosts;
 	unsigned		nsources;
 	struct source		*src;
 	struct token		*t;
@@ -73,17 +77,21 @@ struct tokenlist {
 	int			findent;
 	unsigned		cnt;
 	struct vsb		*fc, *fh, *fi, *ff, *fb;
-	struct vsb		*fm[N_METHODS];
+	struct vsb		*fm[VCL_MET_MAX];
 	VTAILQ_HEAD(, ref)	refs;
 	struct vsb		*sb;
 	int			err;
-	int			nbackend;
+	int			nbackend_host;
+	int			ndirector;
 	VTAILQ_HEAD(, proc)	procs;
 	struct proc		*curproc;
-	struct proc		*mprocs[N_METHODS];
+	struct proc		*mprocs[VCL_MET_MAX];
+
+	VTAILQ_HEAD(, acl_e)	acl;
 
 	unsigned		recnt;
 	unsigned		nhashcount;
+	unsigned		nsockaddr;
 };
 
 enum var_type {
@@ -94,10 +102,9 @@ enum var_type {
 	SIZE,
 	RATE,
 	TIME,
+	RTIME,
 	STRING,
 	IP,
-	HOSTNAME,
-	PORTNAME,
 	HASH,
 	HEADER
 };
@@ -135,7 +142,7 @@ struct var {
 
 struct method {
 	const char		*name;
-	unsigned		returns;
+	unsigned		ret_bitmap;
 	unsigned		bitval;
 };
 
@@ -150,7 +157,17 @@ void vcc_Cond_Ip(const struct var *vp, struct tokenlist *tl);
 void vcc_ParseAction(struct tokenlist *tl);
 
 /* vcc_backend.c */
-void vcc_ParseBackend(struct tokenlist *tl);
+struct fld_spec;
+typedef void parsedirector_f(struct tokenlist *tl,
+    const struct token *t_policy, const struct token *t_dir);
+
+void vcc_ParseDirector(struct tokenlist *tl);
+void vcc_ParseBackendHost(struct tokenlist *tl, int *nbr,
+    const struct token *name, const struct token *qual, int serial);
+struct fld_spec * vcc_FldSpec(struct tokenlist *tl, const char *first, ...);
+void vcc_ResetFldSpec(struct fld_spec *f);
+void vcc_IsField(struct tokenlist *tl, struct token **t, struct fld_spec *fs);
+void vcc_FieldsOk(struct tokenlist *tl, const struct fld_spec *fs);
 
 /* vcc_compile.c */
 extern struct method method_tab[];
@@ -164,13 +181,19 @@ int IsMethod(const struct token *t);
 void TlFree(struct tokenlist *tl, void *p);
 void *TlAlloc(struct tokenlist *tl, unsigned len);
 
+/* vcc_dir_random.c */
+parsedirector_f vcc_ParseRandomDirector;
+
+/* vcc_dir_round_robin.c */
+parsedirector_f vcc_ParseRoundRobinDirector;
+
 /* vcc_obj.c */
-extern struct var vcc_be_vars[];
 extern struct var vcc_vars[];
 
 /* vcc_parse.c */
 void vcc_Parse(struct tokenlist *tl);
 void vcc_RateVal(struct tokenlist *tl);
+void vcc_RTimeVal(struct tokenlist *tl);
 void vcc_TimeVal(struct tokenlist *tl);
 void vcc_SizeVal(struct tokenlist *tl);
 unsigned vcc_UintVal(struct tokenlist *tl);
@@ -182,18 +205,24 @@ int vcc_StringVal(struct tokenlist *tl);
 void vcc_ExpectedStringval(struct tokenlist *tl);
 
 /* vcc_token.c */
+void vcc_Coord(const struct tokenlist *tl, struct vsb *vsb,
+    const struct token *t);
 void vcc_ErrToken(const struct tokenlist *tl, const struct token *t);
 void vcc_ErrWhere(struct tokenlist *tl, const struct token *t);
 void vcc__Expect(struct tokenlist *tl, unsigned tok, int line);
 int vcc_Teq(const struct token *t1, const struct token *t2);
 int vcc_IdIs(const struct token *t, const char *p);
+void vcc_ExpectCid(struct tokenlist *tl);
 void vcc_Lexer(struct tokenlist *tl, struct source *sp);
 void vcc_NextToken(struct tokenlist *tl);
-void vcc__ErrInternal(struct tokenlist *tl, const char *func, unsigned line);
-void vcc_AddToken(struct tokenlist *tl, unsigned tok, const char *b, const char *e);
+void vcc__ErrInternal(struct tokenlist *tl, const char *func,
+    unsigned line);
+void vcc_AddToken(struct tokenlist *tl, unsigned tok, const char *b,
+    const char *e);
 
 /* vcc_var.c */
-struct var *vcc_FindVar(struct tokenlist *tl, const struct token *t, struct var *vl);
+struct var *vcc_FindVar(struct tokenlist *tl, const struct token *t,
+    struct var *vl);
 
 /* vcc_xref.c */
 void vcc_AddDef(struct tokenlist *tl, struct token *t, enum ref_type type);
