@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2007 Linpro AS
+ * Copyright (c) 2006-2008 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,17 +26,20 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vcc_token.c 2034 2007-09-25 08:54:11Z des $
+ * $Id: vcc_token.c 3598 2009-02-05 10:10:58Z tfheen $
  */
 
+#include "config.h"
+
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "vsb.h"
 #include "vqueue.h"
 
+#include "vsb.h"
+
 #include "libvarnish.h"
 #include "vcc_priv.h"
 #include "vcc_compile.h"
@@ -64,32 +67,54 @@ vcc__ErrInternal(struct tokenlist *tl, const char *func, unsigned line)
 	tl->err = 1;
 }
 
-void
-vcc_ErrWhere(struct tokenlist *tl, const struct token *t)
+static void
+vcc_icoord(struct vsb *vsb, const struct token *t, const char **ll)
 {
-	unsigned lin, pos, x, y;
-	const char *p, *l, *f, *b, *e;
+	unsigned lin, pos;
+	const char *p, *b;
 	struct source *sp;
 
 	lin = 1;
 	pos = 0;
 	sp = t->src;
-	f = sp->name;
 	b = sp->b;
-	e = sp->e;
-	for (l = p = b; p < t->b; p++) {
+	if (ll != NULL)
+		*ll = b;
+	for (p = b; p < t->b; p++) {
 		if (*p == '\n') {
 			lin++;
 			pos = 0;
-			l = p + 1;
+			if (ll != NULL)
+				*ll = p + 1;
 		} else if (*p == '\t') {
 			pos &= ~7;
 			pos += 8;
 		} else
 			pos++;
 	}
-	vsb_printf(tl->sb, "(%s Line %d Pos %d)\n", f, lin, pos + 1);
+	vsb_printf(vsb, "(%s Line %d Pos %d)", sp->name, lin, pos + 1);
+}
+
+void
+vcc_Coord(const struct tokenlist *tl, struct vsb *vsb, const struct token *t)
+{
+
+	if (t == NULL)
+		t = tl->t;
+	vcc_icoord(vsb, t, NULL);
+}
+
+void
+vcc_ErrWhere(struct tokenlist *tl, const struct token *t)
+{
+	unsigned x, y;
+	const char *p, *l, *e;
+
+	vcc_icoord(tl->sb, t, &l);
+	vsb_printf(tl->sb, "\n");
+
 	x = y = 0;
+	e = t->src->e;
 	for (p = l; p < e && *p != '\n'; p++) {
 		if (*p == '\t') {
 			y &= ~7;
@@ -184,6 +209,38 @@ vcc_IdIs(const struct token *t, const char *p)
 	return (1);
 }
 
+/*--------------------------------------------------------------------
+ * Check that we have a C-identifier
+ */
+
+static int
+vcc_isCid(const struct token *t)
+{
+	const char *q;
+
+	assert(t->tok == ID);
+	for (q = t->b; q < t->e; q++) {
+		if (!isalnum(*q) && *q != '_')
+			return (0);
+	}
+	return (1);
+}
+
+void
+vcc_ExpectCid(struct tokenlist *tl)
+{
+
+	ExpectErr(tl, ID);
+	ERRCHK(tl);
+	if (vcc_isCid(tl->t))
+		return;
+	vsb_printf(tl->sb, "Identifier ");
+	vcc_ErrToken(tl, tl->t);
+	vsb_printf(tl->sb,
+	    " contains illegal characters, use [0-9a-zA-Z_] only.\n");
+	vcc_ErrWhere(tl, tl->t);
+}
+
 /*--------------------------------------------------------------------
  * Decode %xx in a string
  */
@@ -191,7 +248,7 @@ vcc_IdIs(const struct token *t, const char *p)
 static int8_t
 vcc_xdig(const char c)
 {
-	static const char *xdigit =
+	static const char * const xdigit =
 	    "0123456789abcdef"
 	    "0123456789ABCDEF";
 	const char *p;
@@ -231,7 +288,7 @@ vcc_decstr(struct tokenlist *tl)
 			vcc_ErrWhere(tl, tl->t);
 			return(1);
 		}
-		u = vcc_xdig(p[1]) * 16 + vcc_xdig(p[2]);
+		u = (vcc_xdig(p[1]) * 16 + vcc_xdig(p[2])) & 0xff;
 		if (!isgraph(u)) {
 			vcc_AddToken(tl, CSTR, p, p + 3);
 			vsb_printf(tl->sb,
@@ -266,12 +323,6 @@ vcc_AddToken(struct tokenlist *tl, unsigned tok, const char *b, const char *e)
 	else
 		VTAILQ_INSERT_TAIL(&tl->tokens, t, list);
 	tl->t = t;
-	if (0) {
-		fprintf(stderr, "[%s %.*s] ",
-		    vcl_tnames[tok], PF(t));
-		if (tok == EOI)
-			fprintf(stderr, "\n");
-	}
 }
 
 /*--------------------------------------------------------------------
@@ -311,7 +362,8 @@ vcc_Lexer(struct tokenlist *tl, struct source *sp)
 			if (q < sp->e)
 				continue;
 			vcc_AddToken(tl, EOI, p, p + 2);
-			vsb_printf(tl->sb, "Unterminated /* ... */ comment, starting at\n");
+			vsb_printf(tl->sb,
+			    "Unterminated /* ... */ comment, starting at\n");
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
@@ -328,19 +380,44 @@ vcc_Lexer(struct tokenlist *tl, struct source *sp)
 			for (q = p + 2; q < sp->e; q++) {
 				if (*q == '}' && q[1] == 'C') {
 					vcc_AddToken(tl, CSRC, p, q + 2);
-					p = q + 2;
 					break;
 				}
 			}
-			if (q < sp->e)
+			if (q < sp->e) {
+				p = q + 2;
 				continue;
+			}
 			vcc_AddToken(tl, EOI, p, p + 2);
 			vsb_printf(tl->sb,
 			    "Unterminated inline C source, starting at\n");
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
-	
+
+		/* Recognize long-strings */
+		if (*p == '{' && p[1] == '"') {
+			for (q = p + 2; q < sp->e; q++) {
+				if (*q == '"' && q[1] == '}') {
+					vcc_AddToken(tl, CSTR, p, q + 2);
+					break;
+				}
+			}
+			if (q < sp->e) {
+				p = q + 2;
+				u = tl->t->e - tl->t->b;
+				u -= 4;		/* {" ... "} */
+				tl->t->dec = TlAlloc(tl, u + 1 );
+				AN(tl->t->dec);
+				memcpy(tl->t->dec, tl->t->b + 2, u);
+				tl->t->dec[u] = '\0';
+				continue;
+			}
+			vcc_AddToken(tl, EOI, p, p + 2);
+			vsb_printf(tl->sb,
+			    "Unterminated long-string, starting at\n");
+			vcc_ErrWhere(tl, tl->t);
+			return;
+		}
 
 		/* Match for the fixed tokens (see token.tcl) */
 		u = vcl_fixed_token(p, &q);
